#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.reference.pictogram.interfaces module

"""
from zope.annotation import IAttributeAnnotatable
from zope.container.constraints import containers, contains
from zope.interface import Interface
from zope.schema import Choice, List

from pyams_content.reference.interfaces import IReferenceInfo, IReferenceTable
from pyams_file.schema import I18nImageField
from pyams_i18n.schema import I18nTextLineField


__docformat__ = 'restructuredtext'

from pyams_content import _


PICTOGRAM_VOCABULARY = 'pyams_content.pictograms'
SELECTED_PICTOGRAM_VOCABULARY = 'pyams_content.pictograms.selected'


class IPictogram(IReferenceInfo):
    """Pictogram interface

    Pictograms are managed in a specific reference table to be easily reused by the application
    into any shared content.
    """

    containers('.IPictogramTable')

    image = I18nImageField(title=_("Image"),
                           description=_("Pictogram content"),
                           required=True)

    alt_title = I18nTextLineField(title=_("Accessibility title"),
                                  description=_("Alternate title used to describe image content"),
                                  required=False)

    header = I18nTextLineField(title=_('pictogram-header', default="Header"),
                               description=_("Default header associated with this pictogram"),
                               required=False)


class IPictogramTable(IReferenceTable):
    """Pictograms table interface"""

    contains(IPictogram)


PICTOGRAM_MANAGER_KEY = 'pyams_content.pictogram.manager'


class IPictogramManager(Interface):
    """Pictogram manager interface

    A pictogram manager (typically, a shared tool) is a component which allows selection of a
    set of pictogram which will be available for selection into shared content.
    """

    selected_pictograms = List(title=_("Selected pictograms"),
                               description=_("List of selected pictograms which will be "
                                             "available to shared contents"),
                               required=False,
                               value_type=Choice(vocabulary=PICTOGRAM_VOCABULARY))


class IPictogramManagerTarget(IAttributeAnnotatable):
    """Pictogram manager target interface"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.interface import Interface
from zope.schema import Choice

from pyams_content.shared.common import ISharedContent
from pyams_content.shared.common.interfaces import ISharedToolPortalContext, \
    IWfSharedContentPortalContext
from pyams_content.shared.common.interfaces.types import VISIBLE_DATA_TYPES_VOCABULARY
from pyams_i18n.schema import I18nTextField, I18nTextLineField
from pyams_sequence.interfaces import IInternalReferencesList


__docformat__ = 'restructuredtext'

from pyams_content import _


class IFormManager(ISharedToolPortalContext):
    """Form manager interface"""


FORM_CONTENT_TYPE = 'form'
FORM_CONTENT_NAME = _("Form")


class IWfForm(IWfSharedContentPortalContext, IInternalReferencesList):
    """Form interface"""

    data_type = Choice(title=_("Data type"),
                       description=_("Type of content data"),
                       required=False,
                       vocabulary=VISIBLE_DATA_TYPES_VOCABULARY)

    alt_title = I18nTextLineField(title=_("Title"),
                                  description=_("If set, this title will be displayed in front-office "
                                                "above header and input fields"),
                                  required=False)

    form_header = I18nTextField(title=_("Header"),
                                description=_("If set, this header is displayed just above input fields"),
                                required=False)

    form_legend = I18nTextLineField(title=_("Legend"),
                                    description=_("If set, this legend will be displayed above input fields"),
                                    required=False)


class IWfFormFactory(Interface):
    """Form parent interface"""


class IForm(ISharedContent):
    """Workflow managed form interface"""





#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.glossary.skin.sitemap module

This module defines adapters and views which are used to provide
glossary sitemap.
"""

from pyramid.view import view_config

from pyams_content.component.thesaurus import ITagsManager
from pyams_content.feature.sitemap.interfaces import ISitemapExtension
from pyams_layer.interfaces import IPyAMSUserLayer
from pyams_site.interfaces import ISiteRoot
from pyams_thesaurus.interfaces.thesaurus import IThesaurus
from pyams_utils.adapter import ContextRequestAdapter, adapter_config


__docformat__ = 'restructuredtext'


@adapter_config(name='glossary',
                required=(ISiteRoot, IPyAMSUserLayer),
                provides=ISitemapExtension)
class GlossarySitemapExtension(ContextRequestAdapter):
    """Glossary sitemap extension adapter"""

    @property
    def source(self):
        """Glossary sitemap source"""
        return ITagsManager(self.request.root).glossary


@view_config(name='sitemap.xml',
             context=IThesaurus, request_type=IPyAMSUserLayer,
             renderer='templates/glossary-sitemap.pt')
class GlossarySitemapView:
    """Glossary sitemap view"""

    def __init__(self, request):
        self.request = request

    def __call__(self):
        self.request.response.content_type = 'text/xml'
        return {}



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.video.provider module

This module defines base components which are used to handle external videos
provides by platforms like YouTube, Dailymotion, Vimeo or anything else.
"""

from persistent import Persistent
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_content.component.video import IExternalVideo, IExternalVideoProvider, IExternalVideoSettings, \
    external_video_settings
from pyams_content.component.video.interfaces import PYAMS_VIDEOS_PROVIDERS
from pyams_content.component.video.provider.interfaces import ICustomVideoSettings
from pyams_utils.adapter import adapter_config, get_adapter_weight
from pyams_utils.factory import factory_config
from pyams_utils.registry import get_utilities_for, utility_config
from pyams_utils.request import check_request
from pyams_utils.vocabulary import vocabulary_config

__docformat__ = 'restructuredtext'

from pyams_content import _


@vocabulary_config(name=PYAMS_VIDEOS_PROVIDERS)
class VideoProvidersVocabulary(SimpleVocabulary):
    """Video providers vocabulary"""

    interface = IExternalVideoProvider

    def __init__(self, context):
        request = check_request()
        translate = request.localizer.translate
        utils = sorted(get_utilities_for(self.interface),
                       key=get_adapter_weight)
        terms = [
            SimpleTerm(name, title=getattr(util, 'label', name))
            for name, util in utils
        ]
        super().__init__(terms)


#
# Custom video provider settings
#

@factory_config(ICustomVideoSettings)
class CustomVideoSettings(Persistent):
    """Custom video provider settings"""

    integration_code = FieldProperty(ICustomVideoSettings['integration_code'])


@utility_config(name='custom',
                provides=IExternalVideoProvider)
class CustomVideoProvider:
    """Custom video provider"""

    label = _("Other provider")
    weight = 99

    settings_interface = ICustomVideoSettings


@adapter_config(required=IExternalVideo,
                provides=ICustomVideoSettings)
def custom_video_settings(context):
    """Custom video settings factory"""
    return external_video_settings(context, provider_name='custom')


@adapter_config(required=CustomVideoProvider,
                provides=IExternalVideoSettings)
def custom_video_provider_settings_factory(context):
    """Custom video provider settings factory"""
    return ICustomVideoSettings



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.site.zmi.viewlet module

"""

import locale

from zope.interface import Interface

from pyams_content.shared.common.interfaces import ISharedSite
from pyams_content.zmi.viewlet.toplinks import TopTabsViewletManager
from pyams_i18n.interfaces import II18n
from pyams_utils.registry import get_all_utilities_registered_for
from pyams_viewlet.manager import viewletmanager_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.zmi.viewlet.toplinks import TopMenuViewletManager

__docformat__ = 'restructuredtext'

from pyams_content import _


@viewletmanager_config(name='shared-sites.menu',
                       context=Interface, layer=IAdminLayer,
                       manager=TopTabsViewletManager, weight=10)
class SharedSitesMenu(TopMenuViewletManager):
    """Shared sites menu"""

    label = _("Sites")
    interface = ISharedSite

    def update(self):
        super().update()
        context = self.context
        request = self.request
        parent = self.__parent__
        for site in sorted(get_all_utilities_registered_for(self.interface),
                           key=lambda x: locale.strxfrm(II18n(x).query_attribute('title',
                                                                                 request=request)
                                                        or '')):
            self.add_menu(context, request, parent, site)



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.reference.zmi.table module

This module defines generic components used to handle references tables properties.
"""
from pyramid.view import view_config
from zope.interface import Interface, implementer

from pyams_content.interfaces import MANAGE_REFERENCE_TABLE_PERMISSION
from pyams_content.reference import IReferenceTable
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_table.column import GetAttrColumn
from pyams_table.interfaces import IColumn, IValues
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm
from pyams_zmi.helper.container import delete_container_element
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.form import IPropertiesEditForm
from pyams_zmi.interfaces.viewlet import IMenuHeader, IPropertiesMenu, ISiteManagementMenu
from pyams_zmi.table import I18nColumnMixin, Table, TrashColumn
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_content import _


@viewlet_config(name='properties.menu',
                context=IReferenceTable, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=10,
                permission=VIEW_SYSTEM_PERMISSION)
class ReferenceTablePropertiesMenu(NavigationMenuItem):
    """Reference table properties menu"""

    label = _("Properties")

    href = '#properties.html'


@ajax_form_config(name='properties.html',
                  context=IReferenceTable, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
@implementer(IPropertiesEditForm)
class ReferenceTablePropertiesEditForm(AdminEditForm):
    """Reference table properties edit form"""

    legend = _("Edit table properties")

    fields = Fields(IReferenceTable).omit('__parent__', '__name__')
    edit_permission = MANAGE_REFERENCE_TABLE_PERMISSION


@adapter_config(required=(IReferenceTable, IAdminLayer, Interface, ISiteManagementMenu),
                provides=IMenuHeader)
def reference_table_site_management_menu_header(context, request, view, manager):
    """Reference table site management menu header adapter"""
    return _("Table management")


class ReferenceTableContainerTable(Table):
    """Reference table container table"""

    display_if_empty = True


@adapter_config(required=(IReferenceTable, IAdminLayer, ReferenceTableContainerTable),
                provides=IValues)
class ReferenceTableContainerTableValues(ContextRequestViewAdapter):
    """Reference table container table values adapter"""

    @property
    def values(self):
        """Reference table values getter"""
        yield from self.context.values()


@adapter_config(name='name',
                required=(IReferenceTable, IAdminLayer, ReferenceTableContainerTable),
                provides=IColumn)
class ReferenceTableNameColumn(I18nColumnMixin, GetAttrColumn):
    """Reference table name column"""

    i18n_header = _("Title")
    attr_name = 'title'

    weight = 10

    def get_value(self, obj):
        return II18n(obj).query_attribute(self.attr_name, request=self.request)


@adapter_config(name='trash',
                required=(IReferenceTable, IAdminLayer, ReferenceTableContainerTable),
                provides=IColumn)
class ReferenceTableTrashColumn(TrashColumn):
    """Reference table trash column"""

    permission = MANAGE_REFERENCE_TABLE_PERMISSION


@view_config(name='delete-element.json',
             context=IReferenceTable, request_type=IPyAMSLayer,
             permission=MANAGE_REFERENCE_TABLE_PERMISSION,
             renderer='json', xhr=True)
def delete_table_element(request):
    """Reference table delete view"""
    return delete_container_element(request)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.view.portlet.interfaces module

This module defines interfaces of view items portlet settings.
"""

from collections import OrderedDict

from zope.schema import Bool, Choice, Int
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_content.shared.view.interfaces import VIEW_CONTENT_TYPE
from pyams_content.shared.view.interfaces.query import MergeModes, VIEWS_MERGERS_VOCABULARY
from pyams_i18n.schema import I18nTextLineField
from pyams_portal.interfaces import IPortletSettings
from pyams_sequence.schema import InternalReferencesListField

__docformat__ = 'restructuredtext'

from pyams_content import _


#
# Views display contexts
#

VIEW_DISPLAY_CONTEXT = 'display'
VIEW_CONTENT_CONTEXT = 'content'

VIEW_CONTEXTS = OrderedDict((
    (VIEW_DISPLAY_CONTEXT, _("Display context")),
    (VIEW_CONTENT_CONTEXT, _("Content context"))
))

VIEW_CONTEXT_VOCABULARY = SimpleVocabulary([
    SimpleTerm(v, title=t)
    for v, t in VIEW_CONTEXTS.items()
])

SEARCH_EXCLUDED_ITEMS = 'search.excluded'


#
# Views merge modes
#

VIEW_ITEMS_PORTLET_NAME = 'pyams_content.portlet.view'


class IViewItemsPortletSettings(IPortletSettings):
    """View items portlet settings interface"""

    title = I18nTextLineField(title=_("Title"),
                              required=False)

    views = InternalReferencesListField(title=_("Selected views"),
                                        description=_("Reference to the view(s) from which items are extracted; "
                                                      "you can combine several views together and specify in which "
                                                      "order they should be mixed"),
                                        content_type=VIEW_CONTENT_TYPE,
                                        required=True)

    def get_views(self):
        """Get iterator over selected views"""

    views_context = Choice(title=_("Views context"),
                           description=_("When searching for items, a view receives a \"context\" which is the object "
                                         "from which settings can be extracted; this context can be the \"display\" "
                                         "context or the \"content\" context: when the portlet is used to display the "
                                         "site root, a site manager or a site folder, both are identical; when the "
                                         "portlet is used to display a shared content, the \"content\" context is the "
                                         "displayed content, while the \"display\" context is the container (site "
                                         "root, site manager or site folder) into which content is displayed"),
                           vocabulary=VIEW_CONTEXT_VOCABULARY,
                           default=VIEW_DISPLAY_CONTEXT,
                           required=True)

    views_merge_mode = Choice(title=_("Views merge mode"),
                              description=_("If you select several views, you can select \"merge\" mode, which is "
                                            "the way used to merge items from several views"),
                              vocabulary=VIEWS_MERGERS_VOCABULARY,
                              default=MergeModes.CONCAT.value,
                              required=True)

    def get_merger(self):
        """Get selected views merger utility"""

    limit = Int(title=_("Results count limit"),
                description=_("Maximum number of results that the component may extract from merged views"),
                required=False)

    start = Int(title=_("Starting from..."),
                description=_("You can skip several results if specifying an integer value here..."),
                required=False,
                default=1)

    def get_items(self):
        """Get iterator over items returned by selected views, using selected merger"""

    force_canonical_url = Bool(title=_("Force canonical URL?"),
                               description=_("By default, internal links use a \"relative\" URL, which tries to "
                                             "display link target in the current context; by using a canonical URL, "
                                             "you can display target in it's attachment context (if defined)"),
                               required=False,
                               default=False)

    exclude_from_search = Bool(title=_("Exclude from search results"),
                               description=_("If 'yes', and if this portlet is associated with a search engine in the "
                                             "same page template, items displayed by this portlet will be excluded "
                                             "from search results"),
                               required=True,
                               default=False)

    first_page_only = Bool(title=_("Display on first page only"),
                           description=_("If 'yes', and if this portlet is associated with a search engine in the "
                                         "same page template, view contents will only be displayed on the first page "
                                         "of search results"),
                           required=True,
                           default=False)



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.illustration.skin.interfaces module

This module provides skin interfaces specific to illustrations rendering.
"""

from zope.interface import Attribute, Interface
from zope.schema import Bool

from pyams_content.feature.renderer import IRendererSettings
from pyams_skin.schema import BootstrapThumbnailsSelectionField


__docformat__ = 'restructuredtext'

from pyams_content import _


#
# Illustrations renderers
#

ILLUSTRATION_BEFORE_TITLE = 'before-title'
ILLUSTRATION_BEFORE_BODY = 'before-body'
ILLUSTRATION_AFTER_BODY = 'after-body'


class IIllustrationRenderer(Interface):
    """Illustration renderer interface"""

    position = Attribute("Illustration position related to its attached content")


class IIllustrationDefaultRendererSettings(IRendererSettings):
    """Illustration default renderer settings interface"""

    thumb_selection = BootstrapThumbnailsSelectionField(
        title=_("Image selection"),
        description=_("Illustration can use responsive selections, but you "
                      "can also force selection of another specific format"),
        default_width=12,
        change_width=False,
        required=False)


class IParagraphIllustrationSideRendererSettings(IIllustrationDefaultRendererSettings):
    """Illustration paragraph side renderer settings interface"""

    thumb_selection = BootstrapThumbnailsSelectionField(
        title=_("Image selection"),
        description=_("Illustration can use responsive selections, but you "
                      "can also force selection of another specific format"),
        default_width=5,
        required=False)

    zoom_on_click = Bool(title=_("Zoom on click"),
                         description=_("If 'yes', a click on illustration thumbnail will open a "
                                       "modal window to display image"),
                         required=True,
                         default=True)



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.association module

Associations are components which are used to associate elements to a given context.
These *elements* can be internal or external links, external files...
"""

from persistent import Persistent
from pyramid.events import subscriber
from zope.container.contained import Contained
from zope.interface import implementer
from zope.lifecycleevent import IObjectAddedEvent, IObjectModifiedEvent, IObjectRemovedEvent, \
    ObjectModifiedEvent
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.association.interfaces import IAssociationContainer, \
    IAssociationContainerTarget, IAssociationItem
from pyams_content.shared.common import IWfSharedContent
from pyams_security.interfaces import IViewContextPermissionChecker
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_utils.registry import get_pyramid_registry
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_workflow.content import HiddenContentPublicationInfo
from pyams_workflow.interfaces import IWorkflowPublicationInfo

__docformat__ = 'restructuredtext'


@implementer(IAssociationItem)
class AssociationItem(Persistent, Contained):
    """Base association item persistent class"""

    icon_class = ''
    icon_hint = ''

    visible = FieldProperty(IAssociationItem['visible'])

    def is_visible(self, request=None):
        """Visibility getter"""
        return self.visible

    def get_url(self, request=None, view_name=None):
        """URL getter"""
        return absolute_url(self, request, view_name)


@adapter_config(required=IAssociationItem,
                provides=IViewContextPermissionChecker)
@adapter_config(required=IAssociationContainer,
                provides=IViewContextPermissionChecker)
class AssociationPermissionChecker(ContextAdapter):
    """Association permission checker"""

    @property
    def edit_permission(self):
        """Edit permission getter"""
        parent = get_parent(self.context, IAssociationContainerTarget)
        if parent is not None:
            return IViewContextPermissionChecker(parent).edit_permission
        return None


@subscriber(IObjectAddedEvent, context_selector=IAssociationItem)
@subscriber(IObjectModifiedEvent, context_selector=IAssociationItem)
@subscriber(IObjectRemovedEvent, context_selector=IAssociationItem)
def handle_association_event(event):
    """Handle added association item"""
    content = get_parent(event.object, IWfSharedContent)
    if content is not None:
        get_pyramid_registry().notify(ObjectModifiedEvent(content))


@adapter_config(required=IAssociationItem,
                provides=IWorkflowPublicationInfo)
def association_item_publication_info(context):
    """Association item publication info"""
    if not context.visible:
        return HiddenContentPublicationInfo()
    return None



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.illustration.zmi.paragraph module

This module provides management interface components for illustration paragraphs.
"""

from pyams_content.component.illustration import IBaseIllustration, IIllustrationTarget, \
    IParagraphIllustration
from pyams_content.component.illustration.interfaces import IIllustrationParagraph, \
    ILLUSTRATION_PARAGRAPH_ICON_CLASS, ILLUSTRATION_PARAGRAPH_NAME, ILLUSTRATION_PARAGRAPH_TYPE
from pyams_content.component.paragraph import IParagraphContainer, IParagraphContainerTarget
from pyams_content.component.paragraph.zmi import BaseParagraphAddForm, BaseParagraphAddMenu, \
    BaseParagraphRendererSettingsEditForm, IParagraphContainerBaseTable, \
    IParagraphContainerFullTable
from pyams_content.component.paragraph.zmi.container import ParagraphTitleToolbarItemMixin
from pyams_content.component.paragraph.zmi.interfaces import IParagraphTitleToolbar
from pyams_content.feature.renderer import IRendererSettings
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IFormContent, IFormFields
from pyams_layer.interfaces import IPyAMSLayer
from pyams_portal.zmi.widget import RendererSelectFieldWidget
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_utils.adapter import adapter_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.form import IPropertiesEditForm
from pyams_zmi.interfaces.viewlet import IContextAddingsViewletManager

__docformat__ = 'restructuredtext'


@viewlet_config(name='illustration',
                context=IIllustrationTarget, layer=IAdminLayer,
                view=IParagraphContainerFullTable, manager=IParagraphTitleToolbar,
                weight=10)
@viewlet_config(name='illustration',
                context=IIllustrationParagraph, layer=IAdminLayer,
                view=IParagraphContainerFullTable, manager=IParagraphTitleToolbar,
                weight=10)
class IllustrationParagraphTitleToolbarViewlet(ParagraphTitleToolbarItemMixin):
    """Paragraph illustration marker toolbar viewlet"""

    icon_class = ILLUSTRATION_PARAGRAPH_ICON_CLASS
    icon_hint = ILLUSTRATION_PARAGRAPH_NAME

    target_intf = IBaseIllustration

    def update(self):
        illustration = self.target_intf(self.context, None)
        if (illustration is not None) and illustration.has_data():
            self.counter = 1


@viewlet_config(name='add-illustration-paragraph.menu',
                context=IParagraphContainerTarget, layer=IAdminLayer,
                view=IParagraphContainerBaseTable,
                manager=IContextAddingsViewletManager, weight=60)
class IllustrationParagraphAddMenu(BaseParagraphAddMenu):
    """Illustration paragraph add menu"""

    label = ILLUSTRATION_PARAGRAPH_NAME
    icon_class = ILLUSTRATION_PARAGRAPH_ICON_CLASS

    factory_name = ILLUSTRATION_PARAGRAPH_TYPE
    href = 'add-illustration-paragraph.html'


@ajax_form_config(name='add-illustration-paragraph.html',
                  context=IParagraphContainer, layer=IPyAMSLayer)
class IllustrationParagraphAddForm(BaseParagraphAddForm):
    """Illustration paragraph add form"""

    content_factory = IIllustrationParagraph


@adapter_config(required=(IParagraphContainer, IAdminLayer, IllustrationParagraphAddForm),
                provides=IFormFields)
@adapter_config(required=(IIllustrationParagraph, IAdminLayer, IPropertiesEditForm),
                provides=IFormFields)
def illustration_form_fields(context, request, form):
    """Illustration form fields getter"""
    fields = Fields(IIllustrationParagraph).select('data', 'title', 'alt_title',
                                                   'description', 'author', 'renderer')
    if IIllustrationParagraph.providedBy(context):
        fields['renderer'].widget_factory = RendererSelectFieldWidget
    return fields


@ajax_form_config(name='renderer-settings.html',
                  context=IParagraphIllustration, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class ParagraphIllustrationRendererSettingsEditForm(BaseParagraphRendererSettingsEditForm):
    """Paragraph illustration renderer settings edit form"""


@adapter_config(required=(IParagraphIllustration, IAdminLayer,
                          ParagraphIllustrationRendererSettingsEditForm),
                provides=IFormContent)
def get_paragraph_illustration_renderer_settings_edit_form_content(context, request, form):
    """Paragraph illustration renderer settings edit form content getter"""
    return IRendererSettings(context)



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.review.chat module

Chat messages handlers for contents reviews.
"""

from pyramid.events import subscriber

from pyams_content.feature.review import ICommentAddedEvent, IReviewComments
from pyams_content.interfaces import OWNER_ROLE, READER_ROLE
from pyams_i18n.interfaces import II18n
from pyams_security.interfaces import IProtectedObject
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_utils.request import query_request
from pyams_utils.url import absolute_url


__docformat__ = 'restructuredtext'

from pyams_content import _


try:
    from pyams_chat.interfaces import IChatMessage, IChatMessageHandler
    from pyams_chat.message import ChatMessage
except ImportError:
    pass
else:

    @subscriber(ICommentAddedEvent)
    def handle_new_comment(event):
        """Handle new review comment"""
        request = query_request()
        if request is None:
            return
        content = event.object
        translate = request.localizer.translate
        message = ChatMessage(request=request,
                              context=content,
                              action='notify',
                              category='content.review',
                              source=event.comment.owner,
                              title=translate(_("Review request")),
                              message=translate(_("A new comment was added on "
                                                  "content « {0} »")).format(
                                  II18n(content).query_attribute('title', request=request)),
                              url=absolute_url(content, request, 'admin#review-comments.html'),
                              comments=IReviewComments(content))
        message.send()


    @adapter_config(name='content.review',
                    required=IChatMessage,
                    provides=IChatMessageHandler)
    class ContentReviewMessageHandler(ContextAdapter):
        """Content review message handler"""

        def get_target(self):
            """Message targets getter"""
            context = self.context.context
            principals = set()
            protection = IProtectedObject(context, None)
            if protection is not None:
                principals |= protection.get_principals(READER_ROLE) | \
                              protection.get_principals(OWNER_ROLE)
            comments = self.context.user_data.get('comments')
            if comments is not None:
                principals |= comments.reviewers
            source_id = self.context.source['id']
            if source_id in principals:
                principals.remove(source_id)
            return {
                'principals': tuple(principals)
            }



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.features.renderer module

Content renderers are a very common pattern, which allows to separated *what* will be
rendered from *how* it will be rendered, and the *renderer* is used to specify the *how*:
so, for a given component, you can often choose the renderer which will be used to render
this component.
"""

from pyramid.decorator import reify
from zope.interface import implementer
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_content.feature.renderer.interfaces import IContentRenderer, IRenderedContent, IRendererSettings, \
    RENDERER_SETTINGS_KEY
from pyams_layer.interfaces import IPyAMSLayer
from pyams_portal.interfaces import DEFAULT_RENDERER_NAME, HIDDEN_RENDERER_NAME
from pyams_utils.adapter import adapter_config, get_annotation_adapter
from pyams_utils.factory import get_object_factory
from pyams_utils.request import check_request
from pyams_viewlet.viewlet import BaseContentProvider

__docformat__ = 'restructuredtext'

from pyams_content import _


@implementer(IRenderedContent)
class RenderedContentMixin:
    """Renderer mixin interface"""

    renderer = None
    """Attribute used to store selected content renderer.
    Subclasses should generally override this attribute to define a "Choice" field property based
    on a given renderers vocabulary.
    """

    renderer_interface = IContentRenderer
    """Content renderer interface"""

    def get_renderer(self, request=None):
        """Get rendering adapter based on selected renderer name"""
        if request is None:
            request = check_request()
        renderer = request.registry.queryMultiAdapter((self, request), self.renderer_interface,
                                                      name=self.renderer or '')
        if (renderer is not None) and ('lang' in request.params):
            renderer.language = request.params['lang']
        return renderer


@adapter_config(required=IRenderedContent,
                provides=IContentRenderer)
def rendered_content_renderer_factory(context):
    """Rendered content renderer factory"""
    return context.get_renderer()


@adapter_config(required=IRenderedContent,
                provides=IRendererSettings)
def rendered_content_renderer_settings_factory(context):
    """Rendered content renderer settings factory"""
    renderer = IContentRenderer(context)
    if (renderer is None) or (renderer.settings_interface is None):
        return None
    return get_annotation_adapter(context,
                                  f'{RENDERER_SETTINGS_KEY}::{context.renderer}',
                                  renderer.settings_interface, name='++renderer++')


class RenderersVocabulary(SimpleVocabulary):
    """Renderers vocabulary base class"""

    renderer_interface = IContentRenderer
    """Renderer interface"""

    content_interface = IRenderedContent
    """Interface used to check current context"""

    content_factory = None
    """Factory used to create a new context if current context doesn't implements required
    interface. If no factory is given, vocabulary is looking for default object factory for
    given interface.
    """

    def __init__(self, context=None):
        request = check_request()
        translate = request.localizer.translate
        registry = request.registry
        if not self.content_interface.providedBy(context):
            factory = self.content_factory
            if factory is None:
                factory = get_object_factory(self.content_interface)
            if factory is not None:
                context = factory()
        terms = [
            SimpleTerm(name, title=translate(adapter.label))
            for name, adapter in sorted(registry.getAdapters((context, request),
                                                             self.renderer_interface),
                                        key=lambda x: x[1].weight)
        ]
        super().__init__(terms)


@implementer(IContentRenderer)
class BaseContentRenderer(BaseContentProvider):
    """Base content renderer"""

    label = None
    weight = 0

    settings_interface = None

    @reify
    def settings(self):
        """Renderer settings getter"""
        if self.settings_interface is None:
            return None
        return IRendererSettings(self.context)


@adapter_config(name=HIDDEN_RENDERER_NAME,
                required=(IRenderedContent, IPyAMSLayer),
                provides=IContentRenderer)
class HiddenContentRenderer(BaseContentRenderer):
    """Hidden content renderer"""

    label = _("Hidden content")
    weight = -999

    def render(self, template_name=''):
        """Renderer output"""
        return ''


@adapter_config(name=DEFAULT_RENDERER_NAME,
                required=(IRenderedContent, IPyAMSLayer),
                provides=IContentRenderer)
class DefaultContentRenderer(BaseContentRenderer):
    """Default content renderer"""

    label = _("Default content renderer")



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.cards.skin module

This module defines renderers of Bootstrap cards paragraphs.
"""

from pyams_content.component.cards import ICardsParagraph
from pyams_content.component.cards.skin.interfaces import ICardsParagraphDefaultRendererSettings, \
    ICardsParagraphMasonryRendererSettings
from pyams_content.feature.renderer import BaseContentRenderer, IContentRenderer
from pyams_portal.interfaces import DEFAULT_RENDERER_NAME
from pyams_layer.interfaces import IPyAMSLayer
from pyams_portal.portlets.cards.skin import CardsPortletMasonryRendererSettings, CardsPortletRendererSettings
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import factory_config


__docformat__ = 'restructuredtext'

from pyams_content import _


@factory_config(ICardsParagraphDefaultRendererSettings)
class CardsParagraphDefaultRendererSettings(CardsPortletRendererSettings):
    """Cards paragraph default renderer settings"""


@adapter_config(name=DEFAULT_RENDERER_NAME,
                required=(ICardsParagraph, IPyAMSLayer),
                provides=IContentRenderer)
@template_config(template='templates/cards.pt',
                 layer=IPyAMSLayer)
class CardsParagraphDefaultRenderer(BaseContentRenderer):
    """Cards paragraph default renderer"""

    label = _("Bootstrap cards renderer (default)")

    settings_interface = ICardsParagraphDefaultRendererSettings
    weight = 10


@factory_config(ICardsParagraphMasonryRendererSettings)
class CardsParagraphMasonryRendererSettings(CardsPortletMasonryRendererSettings):
    """Cards paragraph Masonry renderer settings"""


@adapter_config(name='cards::masonry',
                required=(ICardsParagraph, IPyAMSLayer),
                provides=IContentRenderer)
@template_config(template='templates/cards-masonry.pt',
                 layer=IPyAMSLayer)
class CardsParagraphMasonryRenderer(BaseContentRenderer):
    """Cards paragraph masonry renderer"""

    label = _("Bootstrap cards Masonry renderer")

    settings_interface = ICardsParagraphMasonryRendererSettings
    weight = 20



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.workflow.zmi.publication module

"""

from datetime import datetime

from pyams_content.interfaces import MANAGE_SITE_PERMISSION
from pyams_content.shared.site.interfaces import IBaseSiteItem

from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IAJAXFormRenderer
from pyams_layer.interfaces import IPyAMSLayer
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.timezone import tztime
from pyams_viewlet.viewlet import viewlet_config
from pyams_workflow.interfaces import IWorkflowPublicationInfo
from pyams_zmi.form import AdminEditForm
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IPropertiesMenu
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem


__docformat__ = 'restructuredtext'

from pyams_content import _


@viewlet_config(name='workflow-publication.menu',
                context=IBaseSiteItem, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=510,
                permission=MANAGE_SITE_PERMISSION)
class SiteItemPublicationDatesMenu(NavigationMenuItem):
    """Site item publication dates menu"""

    label = _("Publication dates")
    href = '#workflow-publication.html'


@ajax_form_config(name='workflow-publication.html',
                  context=IBaseSiteItem, layer=IPyAMSLayer,
                  permission=MANAGE_SITE_PERMISSION)
class SiteItemPublicationDatesEditForm(AdminEditForm):
    """Site item publication dates edit form"""

    title = _("Workflow management")
    legend = _("Content publication dates")

    fields = Fields(IWorkflowPublicationInfo).select('publication_effective_date',
                                                     'publication_expiration_date')

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        effective_date = self.widgets.get('publication_effective_date')
        if effective_date is not None:
            pub_info = IWorkflowPublicationInfo(self.context)
            if (pub_info is not None) and \
                    (pub_info.first_publication_date is None) and \
                    (pub_info.publication_effective_date is None):
                effective_date.value = tztime(datetime.utcnow())


@adapter_config(required=(IBaseSiteItem, IAdminLayer, SiteItemPublicationDatesEditForm),
                provides=IAJAXFormRenderer)
class SiteItemPublicationDatesEditFormRenderer(ContextRequestViewAdapter):
    """Site item publication dates edit form renderer"""

    def render(self, changes):
        """Changes renderer"""
        if not changes:
            return None
        return {
            'status': 'reload'
        }



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.zmi module

This module defines base management components for all shared contents.
"""

from datetime import datetime
from uuid import uuid4

from pyramid.events import subscriber
from pyramid.location import lineage
from zope.copy import copy
from zope.interface import Interface, implementer
from zope.lifecycleevent import ObjectCreatedEvent
from zope.location import locate

from pyams_content.interfaces import CREATE_CONTENT_PERMISSION, IBaseContent, \
    MANAGE_CONTENT_PERMISSION, MANAGE_SITE_ROOT_PERMISSION, PUBLISH_CONTENT_PERMISSION
from pyams_content.shared.common import IBaseSharedTool, ISharedContent, IWfSharedContent, \
    WfSharedContent
from pyams_content.shared.common.interfaces import IContributorRestrictions, IManagerRestrictions, \
    ISharedTool, IWfSharedContentRoles
from pyams_content.shared.common.interfaces.types import ITypedSharedTool
from pyams_content.shared.common.zmi.interfaces import ISharedContentAddForm, ISharedContentPropertiesMenu
from pyams_content.zmi.properties import PropertiesEditForm
from pyams_content.zmi.widget.seo import I18nSEOTextLineFieldWidget
from pyams_form.ajax import AJAXFormRenderer, ajax_form_config
from pyams_form.button import Buttons, handler
from pyams_form.field import Fields
from pyams_form.interfaces.form import IAJAXFormRenderer, IDataExtractedEvent
from pyams_i18n.interfaces import II18n, II18nManager, INegotiator
from pyams_i18n_views.zmi.manager import I18nManagerLanguagesEditForm
from pyams_layer.interfaces import IFormLayer, IPyAMSLayer
from pyams_security.interfaces import IViewContextPermissionChecker
from pyams_security.interfaces.base import FORBIDDEN_PERMISSION, VIEW_SYSTEM_PERMISSION
from pyams_sequence.interfaces import ISequentialIdInfo
from pyams_skin.interfaces.view import IModalPage
from pyams_skin.interfaces.viewlet import IBreadcrumbItem, IFormHeaderViewletManager
from pyams_skin.schema.button import CloseButton, SubmitButton
from pyams_skin.viewlet.actions import ContextAddAction
from pyams_skin.viewlet.breadcrumb import BreadcrumbItem
from pyams_skin.viewlet.help import AlertMessage
from pyams_skin.viewlet.menu import MenuDivider, MenuItem
from pyams_utils.adapter import ContextAdapter, ContextRequestViewAdapter, adapter_config
from pyams_utils.factory import get_object_factory
from pyams_utils.registry import get_utility
from pyams_utils.request import check_request
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url, generate_url
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_workflow.interfaces import IWorkflow, IWorkflowCommentInfo, IWorkflowInfo, \
    IWorkflowPublicationInfo, IWorkflowState, IWorkflowVersions
from pyams_workflow.versions import WorkflowHistoryItem
from pyams_zmi.form import AdminModalAddForm
from pyams_zmi.helper.event import get_json_widget_refresh_callback
from pyams_zmi.interfaces import IAdminLayer, TITLE_SPAN_BREAK
from pyams_zmi.interfaces.form import IFormTitle
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.interfaces.viewlet import IContentManagementMenu, IContextActionsDropdownMenu, IMenuHeader, \
    INavigationViewletManager, IPropertiesMenu, ISiteManagementMenu, IToolbarViewletManager
from pyams_zmi.table import TableElementEditor
from pyams_zmi.utils import get_object_label
from pyams_zmi.zmi.viewlet.menu import NavigationMenuHeaderDivider, NavigationMenuItem, \
    SiteManagementMenu

__docformat__ = 'restructuredtext'

from pyams_content import _


@viewlet_config(name='add-content.action',
                context=ISharedTool, layer=IAdminLayer,
                manager=IToolbarViewletManager, weight=10,
                permission=CREATE_CONTENT_PERMISSION)
class SharedContentAddAction(ContextAddAction):
    """Shared content add action"""

    @property
    def label(self):
        translate = self.request.localizer.translate
        factory = self.context.shared_content_factory
        return translate(_("Add new: {}")).format(
            translate(factory.factory.content_name).lower())

    href = 'add-shared-content.html'


@ajax_form_config(name='add-shared-content.html',
                  context=ISharedTool, layer=IFormLayer,
                  permission=CREATE_CONTENT_PERMISSION)
@implementer(ISharedContentAddForm)
class SharedContentAddForm(AdminModalAddForm):
    """Shared content add form"""

    @property
    def subtitle(self):
        translate = self.request.localizer.translate
        factory = self.content_factory
        return translate(_("New content: {}")).format(
            translate(factory.factory.content_name).lower())

    legend = _("New content properties")

    @property
    def fields(self):
        factory = get_object_factory(IWfSharedContent, name=self.context.shared_content_type)
        if factory is not None:
            names = ('title', 'data_type', 'notepad') \
                if ITypedSharedTool.providedBy(self.context) else ('title', 'notepad')
            fields = Fields(factory.factory.content_intf).select(*names)
            fields['title'].widget_factory = I18nSEOTextLineFieldWidget
            return fields
        return Fields(Interface)

    _edit_permission = CREATE_CONTENT_PERMISSION

    owner_roles_interface = IWfSharedContentRoles

    @property
    def content_factory(self):
        """Shared content factory"""
        return self.context.shared_content_factory.factory.content_factory

    @property
    def container_target(self):
        """New content container getter"""
        return self.context

    def update_content(self, obj, data):
        """New content update"""
        super().update_content(obj, data)
        # initialize content fields
        lang = get_utility(INegotiator).server_language
        obj.creator = self.request.principal.id
        self.owner_roles_interface(obj).owner = self.request.principal.id
        obj.short_name = obj.title.copy()
        obj.content_url = generate_url(obj.title.get(lang, ''))
        # init content languages
        languages = II18nManager(self.context).languages
        if languages:
            II18nManager(obj).languages = languages.copy()

    def add(self, obj):
        """Add new content to container"""
        factory = self.context.shared_content_factory
        if factory is not None:
            content = factory()
            self.request.registry.notify(ObjectCreatedEvent(content))
            self.__uuid = uuid = str(uuid4())
            self.container_target[uuid] = content
            IWorkflowVersions(content).add_version(obj, None)
            IWorkflowInfo(obj).fire_transition('init', comment=obj.notepad)

    def next_url(self):
        """Redirection URL getter"""
        return absolute_url(self.container_target, self.request,
                            f'{self.__uuid}/++versions++/1/admin')


@adapter_config(required=(ISharedTool, IAdminLayer, SharedContentAddForm),
                provides=IAJAXFormRenderer)
class SharedToolAddFormRenderer(AJAXFormRenderer):
    """Shared tool add form renderer"""

    def render(self, changes):
        """JSON output renderer"""
        return {
            'status': 'redirect',
            'location': self.form.next_url()
        }


#
# Shared content properties edit form
#

@adapter_config(required=IWfSharedContent,
                provides=IViewContextPermissionChecker)
class SharedContentPermissionChecker(ContextAdapter):
    """Shared content form permission checker"""

    @property
    def edit_permission(self):
        """Edit form permission getter"""
        context = self.context
        workflow = IWorkflow(context)
        state = IWorkflowState(context).state
        # forbidden access to all for archived contents
        if state in workflow.readonly_states:
            return FORBIDDEN_PERMISSION
        # only webmaster can update published contents
        if state in workflow.protected_states:
            return MANAGE_SITE_ROOT_PERMISSION
        # webmaster access
        request = check_request()
        if request.has_permission(MANAGE_SITE_ROOT_PERMISSION, context=context):
            return MANAGE_SITE_ROOT_PERMISSION
        principal_id = request.principal.id
        roles = IWfSharedContentRoles(context)
        # access is granted to content's owner and designated contributors or managers
        if principal_id in roles.owner | roles.contributors | roles.managers:
            return MANAGE_CONTENT_PERMISSION
        # restricted manager access
        if state in workflow.manager_states:
            if principal_id in roles.managers:
                return PUBLISH_CONTENT_PERMISSION
            for parent in lineage(context):
                manager_restrictions = IManagerRestrictions(parent, None)
                if manager_restrictions is not None:
                    if manager_restrictions.can_access(context,
                                                       permission=PUBLISH_CONTENT_PERMISSION,
                                                       request=request):
                        return PUBLISH_CONTENT_PERMISSION
            return FORBIDDEN_PERMISSION
        # check if current principal can manage owner's contents
        for parent in lineage(context):
            contrib_restrictions = IContributorRestrictions(parent, None)
            if contrib_restrictions is not None:
                if contrib_restrictions.can_access(context,
                                                   permission=MANAGE_CONTENT_PERMISSION,
                                                   request=request):
                    return MANAGE_CONTENT_PERMISSION
        restrictions = IContributorRestrictions(context)
        if restrictions and restrictions.can_access(context,
                                                    permission=MANAGE_CONTENT_PERMISSION,
                                                    request=request):
            return MANAGE_CONTENT_PERMISSION
        # check if current principal can manage content's due to manager restrictions
        for parent in lineage(self.context):
            manager_restrictions = IManagerRestrictions(parent, None)
            if manager_restrictions is not None:
                if manager_restrictions.can_access(self.context,
                                                   permission=MANAGE_CONTENT_PERMISSION,
                                                   request=request):
                    return MANAGE_CONTENT_PERMISSION
        restrictions = IManagerRestrictions(self.context)
        if restrictions and restrictions.can_access(self.context,
                                                    permission=MANAGE_CONTENT_PERMISSION,
                                                    request=request):
            return MANAGE_CONTENT_PERMISSION
        return FORBIDDEN_PERMISSION


@adapter_config(required=(IWfSharedContent, IAdminLayer, Interface),
                provides=IBreadcrumbItem)
class SharedContentBreadcrumbItem(BreadcrumbItem):
    """Shared content breadcrumb item"""

    @property
    def label(self):
        return II18n(self.context).query_attribute('title', request=self.request)

    view_name = 'admin'


@adapter_config(required=(IWfSharedContent, IAdminLayer, Interface),
                provides=ITableElementEditor)
class SharedContentElementEditor(TableElementEditor):
    """Shared content element editor"""

    view_name = 'admin'
    modal_target = False


@adapter_config(required=(IWfSharedContent, IAdminLayer, Interface, IContentManagementMenu),
                provides=IMenuHeader)
def shared_content_management_menu_header(context, request, view, manager):
    """Shared content management menu header"""
    return request.localizer.translate(_("Content management"))


@viewletmanager_config(name='content.menu',
                       context=IWfSharedContent, layer=IAdminLayer,
                       manager=IContentManagementMenu, weight=10,
                       permission=VIEW_SYSTEM_PERMISSION,
                       provides=IPropertiesMenu)
class SharedContentCompositionMenu(NavigationMenuItem):
    """Shared content composition menu"""

    label = _("Composition")
    icon_class = 'fab fa-dropbox'
    url = '#'


@viewletmanager_config(name='properties.menu',
                       context=IWfSharedContent, layer=IAdminLayer,
                       manager=IPropertiesMenu, weight=10,
                       provides=ISharedContentPropertiesMenu,
                       permission=VIEW_SYSTEM_PERMISSION)
class SharedContentPropertiesMenu(NavigationMenuItem):
    """Shared content properties menu"""

    label = _("Properties")
    href = '#properties.html'


@ajax_form_config(name='properties.html',
                  context=IWfSharedContent, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class SharedContentPropertiesEditForm(PropertiesEditForm):
    """Shared content properties edit form"""

    title = _("Content properties")
    legend = _("Main content properties")

    interface = IWfSharedContent
    fieldnames = ('title', 'short_name', 'content_url', 'header', 'description', 'notepad')

    @property
    def fields(self):
        fields = Fields(self.interface).select(*self.fieldnames)
        if 'title' in fields:
            fields['title'].widget_factory = I18nSEOTextLineFieldWidget
        for name in self.fieldnames:
            if (name in fields) and not getattr(self.context, f'handle_{name}', True):
                fields = fields.omit(name)
        return fields


@adapter_config(required=(IWfSharedContent, IAdminLayer, IModalPage),
                provides=IFormTitle)
def shared_content_modal_page_title(context, request, form):
    """Shared content modal page title"""
    translate = request.localizer.translate
    title = (f"{get_object_label(context, request, form)} "
             f"(v {IWorkflowState(context).version_id})")
    return TITLE_SPAN_BREAK.format(translate(context.content_name), title)


@subscriber(IDataExtractedEvent, form_selector=SharedContentPropertiesEditForm)
def handle_content_properties_data_extraction(event):
    """Automatically set short_name as title"""
    data = event.data
    if not getattr(event.form.context, 'handle_short_name', True):
        data['short_name'] = data['title'].copy()
    if 'content_url' in data:
        data['content_url'] = generate_url(data['content_url'])


@adapter_config(required=(IWfSharedContent, IAdminLayer, SharedContentPropertiesEditForm),
                provides=IAJAXFormRenderer)
class SharedContentPropertiesEditFormRenderer(AJAXFormRenderer):
    """Shared content properties edit form renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if 'title' in changes.get(IBaseContent, ()):
            return {
                'status': 'redirect'
            }
        return super().render(changes)


@adapter_config(name='main',
                required=(IWfSharedContent, IAdminLayer, SharedContentPropertiesEditForm),
                provides=IAJAXFormRenderer)
class SharedContentPropertiesEditFormRenderer(ContextRequestViewAdapter):

    def render(self, changes):
        """AJAX form renderer"""
        if not changes:
            return None
        result = {}
        if 'content_url' in changes.get(IWfSharedContent, ()):
            result['callbacks'] = [
                get_json_widget_refresh_callback(self.view, 'content_url', self.request)
            ]
        return result


@viewlet_config(name='site-manager.menu.divider',
                context=IWfSharedContent, layer=IAdminLayer,
                manager=INavigationViewletManager, weight=199)
class SharedContentSiteManagementMenuDivider(NavigationMenuHeaderDivider):
    """Shared content site management menu divider"""


@viewletmanager_config(name='site-manager.menu',
                       context=IWfSharedContent, layer=IAdminLayer,
                       manager=INavigationViewletManager, weight=200,
                       provides=ISiteManagementMenu)
class SharedContentSiteManagementMenu(SiteManagementMenu):
    """Shared content site management menu header"""

    header = None


@adapter_config(required=(WfSharedContent, IAdminLayer, I18nManagerLanguagesEditForm),
                provides=IViewContextPermissionChecker)
class SharedContentLanguagesEditFormPermissionChecker(ContextRequestViewAdapter):
    """Shared tool languages edit form permission checker"""

    @property
    def edit_permission(self):
        return SharedContentPermissionChecker(self.context).edit_permission


#
# Content duplication
#

@viewlet_config(name='duplicate-content.divider',
                context=IWfSharedContent, layer=IAdminLayer,
                manager=IContextActionsDropdownMenu, weight=49,
                permission=CREATE_CONTENT_PERMISSION)
class SharedContentDuplicateMenuDivider(MenuDivider):
    """Shared content duplication menu divider"""


@viewlet_config(name='duplicate-content.menu',
                context=IWfSharedContent, layer=IAdminLayer,
                manager=IContextActionsDropdownMenu, weight=50,
                permission=CREATE_CONTENT_PERMISSION)
class SharedContentDuplicateMenu(MenuItem):
    """Shared content duplication menu"""

    label = _("Duplicate content")
    icon_class = 'far fa-copy'

    href = 'duplicate-content.html'
    modal_target = True


class ISharedContentDuplicateButtons(Interface):
    """Shared content duplication form buttons"""

    duplicate = SubmitButton(name='duplicate', title=_("Duplicate this content"))
    close = CloseButton(name='close', title=_("Cancel"))


@ajax_form_config(name='duplicate-content.html',
                  context=IWfSharedContent, layer=IPyAMSLayer,
                  permission=CREATE_CONTENT_PERMISSION)
class SharedContentDuplicateForm(AdminModalAddForm):
    """Shared content duplicate form"""

    subtitle = _("Duplicate content")
    legend = _("New content properties")

    fields = Fields(IWfSharedContent).select('title') + \
        Fields(IWorkflowCommentInfo)
    buttons = Buttons(ISharedContentDuplicateButtons)

    _edit_permission = CREATE_CONTENT_PERMISSION

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        title = self.widgets.get('title')
        if title is not None:
            title.value = self.context.title.copy()
            title.update()

    @handler(buttons['duplicate'])
    def handle_duplicate(self, action):
        super().handle_add(self, action)

    def create_and_add(self, data):
        data = data.get(self, data)
        request = self.request
        registry = request.registry
        # initialize new content
        content = get_parent(self.context, ISharedContent)
        new_content = content.__class__()
        registry.notify(ObjectCreatedEvent(new_content))
        container = get_parent(content, IBaseSharedTool)
        container[str(uuid4())] = new_content
        # initialize new version
        new_version = copy(self.context)
        registry.notify(ObjectCreatedEvent(new_version))
        locate(new_version, self.context.__parent__)  # locate new version for traversing to work...
        new_version.title = data['title']
        if getattr(new_version, 'handle_short_name', True):
            new_version.short_name = new_version.title.copy()
        if getattr(new_version, 'handle_content_url', True):
            lang = get_utility(INegotiator).server_language
            new_version.content_url = generate_url(new_version.title.get(lang, ''))
        new_version.creator = request.principal.id
        new_version.modifiers = set()
        roles = IWfSharedContentRoles(new_version)
        roles.owner = request.principal.id
        # store new version
        translate = self.request.localizer.translate
        workflow = IWorkflow(new_content)
        IWorkflowVersions(new_content).add_version(new_version, workflow.initial_state)
        # reset publication info
        pub_info = IWorkflowPublicationInfo(new_version, None)
        if pub_info is not None:
            pub_info.reset(complete=True)
        # update new version history
        source_state = IWorkflowState(self.context)
        state = IWorkflowState(new_version)
        state.history.clear()
        history = WorkflowHistoryItem(date=datetime.utcnow(),
                                      principal=request.principal.id,
                                      comment=translate(
                                          _("Clone created from version {source} of {oid} "
                                            "(in « {state} » state)")).format(
                                          source=source_state.version_id,
                                          oid=ISequentialIdInfo(self.context).get_short_oid(),
                                          state=translate(IWorkflow(self.context).get_state_label(
                                              source_state.state))))
        state.history.append(history)
        history = WorkflowHistoryItem(date=datetime.utcnow(),
                                      target_state=workflow.initial_state,
                                      principal=request.principal.id,
                                      comment=data.get('comment'))
        state.history.append(history)
        return new_version


@adapter_config(required=(IWfSharedContent, IAdminLayer, SharedContentDuplicateForm),
                provides=IAJAXFormRenderer)
class SharedContentDuplicateFormRenderer(ContextRequestViewAdapter):
    """Shared content duplicate form renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if changes is None:
            return None
        return {
            'status': 'redirect',
            'location': absolute_url(changes, self.request, 'admin')
        }


@viewlet_config(name='duplication-help',
                context=IWfSharedContent, layer=IAdminLayer, view=SharedContentDuplicateForm,
                manager=IFormHeaderViewletManager, weight=10)
class SharedContentDuplicateHelp(AlertMessage):
    """Shared content duplicate help"""

    _message = _("You are going to duplicate a whole content.<br />"
                 "The new copy is going to be created in 'draft' mode, so that you can modify "
                 "it before publication.<br />"
                 "A new unique number is also going to be assigned to it. This number will be "
                 "shared by all content's versions.")
    message_renderer = 'markdown'



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.root.zmi.dashboard module

This module defines management components for site root dashboard.
"""

from hypatia.catalog import CatalogQuery
from hypatia.interfaces import ICatalog
from hypatia.query import And, Any, Eq, Or
from pyramid.decorator import reify
from pyramid.interfaces import IView
from zope.interface import Interface, implementer
from zope.intid import IIntIds
from zope.schema.vocabulary import getVocabularyRegistry

from pyams_catalog.query import CatalogResultSet
from pyams_content.shared.common import IBaseSharedTool, SHARED_CONTENT_TYPES_VOCABULARY
from pyams_content.shared.common.zmi.dashboard import BaseSharedToolDashboardSingleView, \
    DashboardTable, BaseSharedToolDashboardView, SharedToolAllInterventionsMenu, \
    SharedToolArchivedContentsMenu, SharedToolArchivedContentsView, \
    SharedToolDashboardManagerWaitingValues, SharedToolDashboardManagerWaitingView, \
    SharedToolDashboardMenu, SharedToolDashboardOwnerModifiedView, \
    SharedToolDashboardOwnerWaitingView, SharedToolLastModificationsView, \
    SharedToolLastModifiedMenu, SharedToolLastPublicationsMenu, SharedToolLastPublicationsView, \
    SharedToolMissingContentWarning, SharedToolMyDashboardMenu, SharedToolPreparationsMenu, \
    SharedToolPreparationsView, SharedToolPublicationsMenu, SharedToolPublicationsView, \
    SharedToolRetiredContentsMenu, SharedToolRetiredContentsView, SharedToolSubmissionsMenu, \
    SharedToolSubmissionsView
from pyams_content.zmi.dashboard import DashboardContentTypeColumn
from pyams_content.zmi.interfaces import IAllDashboardMenu, IDashboardTable, IMyDashboardMenu, \
    ISiteRootDashboardContentType
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_site.interfaces import ISiteRoot
from pyams_table.interfaces import IColumn, IValues
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.list import unique_iter
from pyams_utils.registry import get_all_utilities_registered_for, get_utility
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_workflow.interfaces import IWorkflow
from pyams_workflow.versions import get_last_version_in_state
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.table import IInnerTable
from pyams_zmi.interfaces.viewlet import IContentManagementMenu, IMenuHeader
from pyams_zmi.table import MultipleTablesAdminView


__docformat__ = 'restructuredtext'

from pyams_content import _


class BaseSiteRootDashboardTable(DashboardTable):
    """Base site root dashboard table"""


class BaseSiteRootDashboardView(BaseSharedToolDashboardView):
    """Base site root dashboard view"""


class BaseSiteRootDashboardSingleView(BaseSharedToolDashboardSingleView):
    """Base site root dashboard view with single table"""


@adapter_config(name='content-type',
                required=(ISiteRoot, IAdminLayer, IDashboardTable),
                provides=IColumn)
class SiteRootContentTypeColumn(DashboardContentTypeColumn):
    """Site root dashboard content type column"""

    i18n_header = _("Content type")
    interface = ISiteRootDashboardContentType


#
# Main shared tool dashboard
#

@adapter_config(required=(ISiteRoot, IAdminLayer, Interface, IContentManagementMenu),
                provides=IMenuHeader)
def site_root_menu_header(context, request, view, menu):
    """Site root menu header"""
    return request.localizer.translate(_("Users activity"))


@viewlet_config(name='dashboard.menu',
                context=ISiteRoot, layer=IAdminLayer,
                manager=IContentManagementMenu, weight=5,
                permission=VIEW_SYSTEM_PERMISSION)
class SiteRootDashboardMenu(SharedToolDashboardMenu):
    """Site root dashboard menu"""


@pagelet_config(name='dashboard.html',
                context=ISiteRoot, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class SiteRootDashboardView(MultipleTablesAdminView):
    """Site root dashboard view"""

    header_label = _("My dashboard")
    table_label = _("My dashboard")


@adapter_config(name='no-content-warning',
                required=(ISiteRoot, IAdminLayer, SiteRootDashboardView),
                provides=IInnerTable, force_implements=False)
class SiteRootMissingContentWarning(SharedToolMissingContentWarning):
    """Site root missing content warning"""


#
# Manager waiting contents
#

class SiteRootDashboardManagerWaitingTable(BaseSiteRootDashboardTable):
    """Site root dashboard manager waiting table"""

    @reify
    def id(self):
        return f'{super().id}_waiting'


@adapter_config(name='manager-waiting',
                required=(ISiteRoot, IAdminLayer, SiteRootDashboardView),
                provides=IInnerTable)
class SiteRootDashboardManagerWaitingView(SharedToolDashboardManagerWaitingView):
    """Site root dashboard manager waiting view"""

    table_class = SiteRootDashboardManagerWaitingTable


@adapter_config(required=(ISiteRoot, IAdminLayer, SiteRootDashboardManagerWaitingTable),
                provides=IValues)
class SiteRootDashboardManagerWaitingValues(SharedToolDashboardManagerWaitingValues):
    """Site root dashboard waiting values adapter"""

    @property
    def values(self):
        """Table values getter"""
        intids = get_utility(IIntIds)
        catalog = get_utility(ICatalog)
        vocabulary = getVocabularyRegistry().get(self.context, SHARED_CONTENT_TYPES_VOCABULARY)
        params = None
        for tool in get_all_utilities_registered_for(IBaseSharedTool):
            workflow = IWorkflow(tool)
            query = And(Eq(catalog['parents'], intids.register(tool)),
                        Any(catalog['content_type'], vocabulary.by_value.keys()),
                        Any(catalog['workflow_state'], workflow.waiting_states))
            params = params | query if params else query
        yield from filter(
            self.check_access,
            unique_iter(
                map(get_last_version_in_state,
                    CatalogResultSet(CatalogQuery(catalog).query(params,
                                                                 sort_index='modified_date')))))


#
# Last owned contents waiting for action
#

class SiteRootDashboardOwnerWaitingTable(BaseSiteRootDashboardTable):
    """Table of owned contents waiting for action"""

    @reify
    def id(self):
        return f'{super().id}_owner_waiting'


@adapter_config(name='owner-waiting',
                required=(ISiteRoot, IAdminLayer, SiteRootDashboardView),
                provides=IInnerTable)
class SiteRootDashboardOwnerWaitingView(SharedToolDashboardOwnerWaitingView):
    """View of owned contents waiting for action"""

    table_class = SiteRootDashboardOwnerWaitingTable


@adapter_config(required=(ISiteRoot, IAdminLayer, SiteRootDashboardOwnerWaitingTable),
                provides=IValues)
class SiteRootDashboardOwnerWaitingValues(ContextRequestViewAdapter):
    """Site root dashboard waiting owned contents values adapter"""

    @property
    def values(self):
        """Table values getter"""
        principal_id = self.request.principal.id
        intids = get_utility(IIntIds)
        catalog = get_utility(ICatalog)
        vocabulary = getVocabularyRegistry().get(self.context, SHARED_CONTENT_TYPES_VOCABULARY)
        params = None
        for tool in get_all_utilities_registered_for(IBaseSharedTool):
            workflow = IWorkflow(tool)
            query = And(Eq(catalog['parents'], intids.register(tool)),
                        Any(catalog['content_type'], vocabulary.by_value.keys()),
                        Any(catalog['workflow_state'], workflow.waiting_states),
                        Eq(catalog['workflow_principal'], principal_id))
            params = params | query if params else query
        yield from unique_iter(
            map(get_last_version_in_state,
                CatalogResultSet(CatalogQuery(catalog).query(params,
                                                             sort_index='modified_date'))))


#
# Last owner modified contents
#

class SiteRootDashboardOwnerModifiedTable(BaseSiteRootDashboardTable):
    """Site root dashboard owner modified table"""

    @reify
    def id(self):
        return f'{super().id}_modified'


@adapter_config(name='owner-modified',
                required=(ISiteRoot, IAdminLayer, SiteRootDashboardView),
                provides=IInnerTable)
class SiteRootDashboardOwnerModifiedView(SharedToolDashboardOwnerModifiedView):
    """Site root dashboard owner modified view"""

    table_class = SiteRootDashboardOwnerModifiedTable


@adapter_config(required=(ISiteRoot, IAdminLayer, SiteRootDashboardOwnerModifiedTable),
                provides=IValues)
class SiteRootDashboardOwnerModifiedValues(ContextRequestViewAdapter):
    """Site root dashboard owner modified adapter"""

    @property
    def values(self):
        """Table values getter"""
        principal_id = self.request.principal.id
        intids = get_utility(IIntIds)
        catalog = get_utility(ICatalog)
        vocabulary = getVocabularyRegistry().get(self.context, SHARED_CONTENT_TYPES_VOCABULARY)
        params = None
        for tool in get_all_utilities_registered_for(IBaseSharedTool):
            query = And(Eq(catalog['parents'], intids.register(tool)),
                        Any(catalog['content_type'], vocabulary.by_value.keys()),
                        Or(Eq(catalog['role:owner'], principal_id),
                           Eq(catalog['role:contributor'], principal_id)))
            params = params | query if params else query
        yield from unique_iter(
            map(get_last_version_in_state,
                CatalogResultSet(CatalogQuery(catalog).query(params,
                                                             limit=50,
                                                             sort_index='modified_date',
                                                             reverse=True))))


#
# All my contents menu
#

@viewletmanager_config(name='my-contents.menu',
                       context=ISiteRoot, layer=IAdminLayer,
                       manager=IContentManagementMenu, weight=10,
                       permission=VIEW_SYSTEM_PERMISSION,
                       provides=IMyDashboardMenu)
class SiteRootMyDashboardMenu(SharedToolMyDashboardMenu):
    """Site root 'my contents' dashboard menu"""


#
# My preparations
# Dashboard of owned and modified contents which can be updated
#

@viewlet_config(name='my-preparations.menu',
                context=ISiteRoot, layer=IAdminLayer,
                manager=IMyDashboardMenu, weight=5,
                permission=VIEW_SYSTEM_PERMISSION)
class SiteRootPreparationsMenu(SharedToolPreparationsMenu):
    """Site root preparations dashboard menu"""


@implementer(IView)
class SiteRootPreparationsTable(BaseSiteRootDashboardTable):
    """Site root preparations table"""


@adapter_config(required=(ISiteRoot, IAdminLayer, SiteRootPreparationsTable),
                provides=IValues)
class SiteRootPreparationsValues(ContextRequestViewAdapter):
    """Site root preparations values adapter"""

    @property
    def values(self):
        principal_id = self.request.principal.id
        intids = get_utility(IIntIds)
        catalog = get_utility(ICatalog)
        vocabulary = getVocabularyRegistry().get(self.context, SHARED_CONTENT_TYPES_VOCABULARY)
        params = None
        for tool in get_all_utilities_registered_for(IBaseSharedTool):
            workflow = IWorkflow(tool)
            query = And(Eq(catalog['parents'], intids.register(self.context)),
                        Any(catalog['content_type'], vocabulary.by_value.keys()),
                        Or(Eq(catalog['role:owner'], principal_id),
                           Eq(catalog['role:contributor'], principal_id)),
                        Eq(catalog['workflow_state'], workflow.initial_state))
            params = params | query if params else query
        yield from unique_iter(
            CatalogResultSet(CatalogQuery(catalog).query(params,
                                                         sort_index='modified_date',
                                                         reverse=True)))


@pagelet_config(name='my-preparations.html',
                context=ISiteRoot, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class SiteRootPreparationsView(SharedToolPreparationsView):
    """Site root preparations view"""

    table_class = SiteRootPreparationsTable


#
# My submissions
# Dashboard of contents waiting for manager action
#

@viewlet_config(name='my-submissions.menu',
                context=ISiteRoot, layer=IAdminLayer,
                manager=IMyDashboardMenu, weight=10,
                permission=VIEW_SYSTEM_PERMISSION)
class SiteRootSubmissionsMenu(SharedToolSubmissionsMenu):
    """Site root submissions dashboard menu"""


@implementer(IView)
class SiteRootSubmissionsTable(BaseSiteRootDashboardTable):
    """Site root submissions table"""


@adapter_config(required=(ISiteRoot, IAdminLayer, SiteRootSubmissionsTable),
                provides=IValues)
class SiteRootSubmissionsValues(ContextRequestViewAdapter):
    """Site root submissions values adapter"""

    @property
    def values(self):
        context = self.context
        principal_id = self.request.principal.id
        intids  =get_utility(IIntIds)
        catalog = get_utility(ICatalog)
        vocabulary = getVocabularyRegistry().get(context, SHARED_CONTENT_TYPES_VOCABULARY)
        params = None
        for tool in get_all_utilities_registered_for(IBaseSharedTool):
            workflow = IWorkflow(tool)
            query = And(Eq(catalog['parents'], intids.register(context)),
                        Any(catalog['content_type'], vocabulary.by_value.keys()),
                        Or(Eq(catalog['role:owner'], principal_id),
                           Eq(catalog['role:contributor'], principal_id)),
                        Any(catalog['workflow_state'], workflow.waiting_states))
            params = params | query if params else query
        yield from unique_iter(
            CatalogResultSet(CatalogQuery(catalog).query(params,
                                                         sort_index='modified_date',
                                                         reverse=True)))


@pagelet_config(name='my-submissions.html',
                context=ISiteRoot, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class SiteRootSubmissionsView(SharedToolSubmissionsView):
    """Site root submissions view"""

    table_class = SiteRootSubmissionsTable


#
# My publications
# Dashboard of published contents
#

@viewlet_config(name='my-publications.menu',
                context=ISiteRoot, layer=IAdminLayer,
                manager=IMyDashboardMenu, weight=15,
                permission=VIEW_SYSTEM_PERMISSION)
class SiteRootPublicationsMenu(SharedToolPublicationsMenu):
    """Site root publication dashboard menu"""


@implementer(IView)
class SiteRootPublicationsTable(BaseSiteRootDashboardTable):
    """Site root publications table"""


@adapter_config(required=(ISiteRoot, IAdminLayer, SiteRootPublicationsTable),
                provides=IValues)
class SiteRootPublicationsValues(ContextRequestViewAdapter):
    """Site root publications values adapter"""

    @property
    def values(self):
        context = self.context
        principal_id = self.request.principal.id
        intids = get_utility(IIntIds)
        catalog = get_utility(ICatalog)
        vocabulary = getVocabularyRegistry().get(context, SHARED_CONTENT_TYPES_VOCABULARY)
        params = None
        for tool in get_all_utilities_registered_for(IBaseSharedTool):
            workflow = IWorkflow(tool)
            query = And(Eq(catalog['parents'], intids.register(self.context)),
                        Any(catalog['content_type'], vocabulary.by_value.keys()),
                        Or(Eq(catalog['role:owner'], principal_id),
                           Eq(catalog['role:contributor'], principal_id)),
                        Any(catalog['workflow_state'], workflow.published_states))
            params = params | query if params else query
        yield from unique_iter(
            CatalogResultSet(CatalogQuery(catalog).query(params,
                                                         sort_index='modified_date',
                                                         reverse=True)))


@pagelet_config(name='my-publications.html',
                context=ISiteRoot, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class SiteRootPublicationsView(SharedToolPublicationsView):
    """Site root publications view"""

    table_class = SiteRootPublicationsTable


#
# My retired contents
# Dashboard of retired contents
#

@viewlet_config(name='my-retired-contents.menu',
                context=ISiteRoot, layer=IAdminLayer,
                manager=IMyDashboardMenu, weight=20,
                permission=VIEW_SYSTEM_PERMISSION)
class SiteRootRetiredContentsMenu(SharedToolRetiredContentsMenu):
    """Site root retired contents dashboard menu"""


@implementer(IView)
class SiteRootRetiredContentsTable(BaseSiteRootDashboardTable):
    """Site root retired contents table"""


@adapter_config(required=(ISiteRoot, IAdminLayer, SiteRootRetiredContentsTable),
                provides=IValues)
class SiteRootRetiredContentsValues(ContextRequestViewAdapter):
    """Site root retired contents values adapter"""

    @property
    def values(self):
        context = self.context
        principal_id = self.request.principal.id
        intids = get_utility(IIntIds)
        catalog = get_utility(ICatalog)
        vocabulary = getVocabularyRegistry().get(context, SHARED_CONTENT_TYPES_VOCABULARY)
        params = None
        for tool in get_all_utilities_registered_for(IBaseSharedTool):
            workflow = IWorkflow(tool)
            query = And(Eq(catalog['parents'], intids.register(context)),
                        Any(catalog['content_type'], vocabulary.by_value.keys()),
                        Or(Eq(catalog['role:owner'], principal_id),
                           Eq(catalog['role:contributor'], principal_id)),
                        Any(catalog['workflow_state'], workflow.retired_states))
            params = params | query if params else query
        yield from unique_iter(
            CatalogResultSet(CatalogQuery(catalog).query(params,
                                                         sort_index='modified_date',
                                                         reverse=True)))


@pagelet_config(name='my-retired-contents.html',
                context=ISiteRoot, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class SiteRootRetiredContentsView(SharedToolRetiredContentsView):
    """Site root retired contents view"""

    table_class = SiteRootRetiredContentsTable


#
# My archived contents
# Dashboard of archived contents
#

@viewlet_config(name='my-archived-contents.menu',
                context=ISiteRoot, layer=IAdminLayer,
                manager=IMyDashboardMenu, weight=25,
                permission=VIEW_SYSTEM_PERMISSION)
class SiteRootArchivedContentsMenu(SharedToolArchivedContentsMenu):
    """Site root archived contents dashboard menu"""


@implementer(IView)
class SiteRootArchivedContentsTable(BaseSiteRootDashboardTable):
    """Site root archived contents table"""


@adapter_config(required=(ISiteRoot, IAdminLayer, SiteRootArchivedContentsTable),
                provides=IValues)
class SiteRootArchivedContentsValues(ContextRequestViewAdapter):
    """Site root archived contents values adapter"""

    @property
    def values(self):
        context = self.context
        principal_id = self.request.principal.id
        intids = get_utility(IIntIds)
        catalog = get_utility(ICatalog)
        vocabulary = getVocabularyRegistry().get(context, SHARED_CONTENT_TYPES_VOCABULARY)
        params = None
        for tool in get_all_utilities_registered_for(IBaseSharedTool):
            workflow = IWorkflow(tool)
            query = And(Eq(catalog['parents'], intids.register(context)),
                        Any(catalog['content_type'], vocabulary.by_value.keys()),
                        Or(Eq(catalog['role:owner'], principal_id),
                           Eq(catalog['role:contributor'], principal_id)),
                        Any(catalog['workflow_state'], workflow.archived_states))
            params = params | query if params else query
        yield from unique_iter(
            CatalogResultSet(CatalogQuery(catalog).query(params,
                                                         sort_index='modified_date',
                                                         reverse=True)))


@pagelet_config(name='my-archived-contents.html',
                context=ISiteRoot, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class SiteRootArchivedContentsView(SharedToolArchivedContentsView):
    """Site root archived contents view"""

    table_class = SiteRootArchivedContentsTable


#
# All interventions menu
#

@viewletmanager_config(name='my-interventions.menu',
                       context=ISiteRoot, layer=IAdminLayer,
                       manager=IContentManagementMenu, weight=20,
                       permission=VIEW_SYSTEM_PERMISSION,
                       provides=IAllDashboardMenu)
class SiteRootAllInterventionsMenu(SharedToolAllInterventionsMenu):
    """Site root 'all interventions' dashboard menu"""


#
# Last published contents
#

@viewlet_config(name='last-published.menu',
                context=ISiteRoot, layer=IAdminLayer,
                manager=IAllDashboardMenu, weight=25,
                permission=VIEW_SYSTEM_PERMISSION)
class SiteRootLastPublicationsMenu(SharedToolLastPublicationsMenu):
    """Site root modified contents dashboard menu"""


@implementer(IView)
class SiteRootLastPublicationsTable(BaseSiteRootDashboardTable):
    """Site root dashboard last publications table"""


@adapter_config(required=(ISiteRoot, IAdminLayer, SiteRootLastPublicationsTable),
                provides=IValues)
class SiteRootLastPublicationsValues(ContextRequestViewAdapter):
    """Site root publications values adapter"""

    @property
    def values(self):
        intids = get_utility(IIntIds)
        catalog = get_utility(ICatalog)
        vocabulary = getVocabularyRegistry().get(self.context, SHARED_CONTENT_TYPES_VOCABULARY)
        params = None
        for tool in get_all_utilities_registered_for(IBaseSharedTool):
            workflow = IWorkflow(tool)
            query = And(Eq(catalog['parents'], intids.register(self.context)),
                        Any(catalog['content_type'], vocabulary.by_value.keys()),
                        Any(catalog['workflow_state'], workflow.published_states))
            params = params | query if params else query
        yield from unique_iter(
            CatalogResultSet(CatalogQuery(catalog).query(params,
                                                         limit=50,
                                                         sort_index='modified_date',
                                                         reverse=True)))


@pagelet_config(name='last-published.html',
                context=ISiteRoot, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class SiteRootLastPublicationsView(SharedToolLastPublicationsView):
    """Site root last publications view"""

    table_class = SiteRootLastPublicationsTable


#
# Last modified contents
#

@viewlet_config(name='last-modified.menu',
                context=ISiteRoot, layer=IAdminLayer,
                manager=IAllDashboardMenu, weight=30,
                permission=VIEW_SYSTEM_PERMISSION)
class SiteRootLastModifiedMenu(SharedToolLastModifiedMenu):
    """Site root modified contents dashboard menu"""


@implementer(IView)
class SiteRootLastModificationsTable(BaseSiteRootDashboardTable):
    """Site root dashboard last modifications table"""


@adapter_config(required=(ISiteRoot, IAdminLayer, SiteRootLastModificationsTable),
                provides=IValues)
class SiteRootLastModificationsValues(ContextRequestViewAdapter):
    """Site root modifications values adapter"""

    @property
    def values(self):
        catalog = get_utility(ICatalog)
        vocabulary = getVocabularyRegistry().get(self.context, SHARED_CONTENT_TYPES_VOCABULARY)
        params = Any(catalog['content_type'], vocabulary.by_value.keys())
        yield from unique_iter(
            CatalogResultSet(CatalogQuery(catalog).query(params,
                                                         limit=50,
                                                         sort_index='modified_date',
                                                         reverse=True)))


@pagelet_config(name='last-modified.html',
                context=ISiteRoot, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class SiteRootLastModificationsView(SharedToolLastModificationsView):
    """Site root last modifications view"""

    table_class = SiteRootLastModificationsTable



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.review.zmi module

Management components for content review.
"""

from pyramid.renderers import render
from pyramid.view import view_config
from zope.interface import Interface
from zope.schema import Bool, Text

from pyams_content.feature.review import IReviewComment, IReviewComments, IReviewManager, \
    IReviewTarget
from pyams_content.interfaces import COMMENT_CONTENT_PERMISSION, MANAGE_CONTENT_PERMISSION
from pyams_content.shared.common import IWfSharedContentRoles
from pyams_content.zmi import content_js
from pyams_form.ajax import ajax_form_config
from pyams_form.button import Buttons, handler
from pyams_form.field import Fields
from pyams_form.interfaces.form import IAJAXFormRenderer
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces import ISecurityManager
from pyams_security.schema import PrincipalsSetField
from pyams_security.utility import get_principal
from pyams_skin.schema.button import CloseButton, SubmitButton
from pyams_skin.viewlet.menu import MenuItem
from pyams_template.template import template_config
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.date import format_datetime, get_age
from pyams_utils.factory import get_object_factory
from pyams_utils.fanstatic import get_resource_path
from pyams_utils.registry import get_utility
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminModalAddForm
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.profile import IUserProfile
from pyams_zmi.interfaces.viewlet import IContextActionsDropdownMenu, IContentManagementMenu
from pyams_zmi.view import InnerAdminView
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem


__docformat__ = 'restructuredtext'

from pyams_content import _


@viewlet_config(name='content-review.menu',
                context=IReviewTarget, layer=IAdminLayer,
                manager=IContextActionsDropdownMenu, weight=10,
                permission=MANAGE_CONTENT_PERMISSION)
class ContentReviewMenu(MenuItem):
    """Content review menu"""

    label = _("Ask for content review")
    icon_class = 'fas fa-eye'

    href = 'content-review.html'
    modal_target = True


class IContentReviewFormFields(Interface):
    """Content review interface"""

    reviewers = PrincipalsSetField(title=_("Sought principals"),
                                   description=_("List of principals from which a review is "
                                                 "requested"),
                                   required=True)

    comment = Text(title=_("Comment"),
                   description=_("Comment associated with this request"),
                   required=True)

    notify_all = Bool(title=_("Notify all reviewers"),
                      description=_("If 'yes', selected reviewers will be notified by mail of "
                                    "your request, even if they were already members of the "
                                    "reviewers group. Otherwise, only new reviewers will be "
                                    "notified"),
                      default=True,
                      required=True)


class IContentReviewButtons(Interface):
    """Shared content review form buttons"""

    review = SubmitButton(name='review',
                          title=_("Ask for content review"))

    close = CloseButton(name='close',
                        title=_("Cancel"))


@ajax_form_config(name='content-review.html',
                  context=IReviewTarget, layer=IPyAMSLayer,
                  permission=MANAGE_CONTENT_PERMISSION)
class ContentReviewAskForm(AdminModalAddForm):
    """Content review add form"""

    subtitle = _("Content review")
    legend = _("Content review request")

    fields = Fields(IContentReviewFormFields)
    buttons = Buttons(IContentReviewButtons)

    _edit_permission = MANAGE_CONTENT_PERMISSION

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        comment = self.widgets.get('comment')
        if comment is not None:
            comment.rows = 10

    @handler(buttons['review'])
    def handle_review(self, action):
        """Review button handler"""
        self.handle_add(self, action)

    def create_and_add(self, data):
        data = data.get(self, data)
        manager = IReviewManager(self.context, None)
        if manager is not None:
            return manager.ask_review(request=self.request, **data)


@adapter_config(required=(IReviewTarget, IAdminLayer, ContentReviewAskForm),
                provides=IAJAXFormRenderer)
class ContentReviewAskFormRenderer(ContextRequestViewAdapter):
    """Content review ask form renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if changes is None:
            return None
        translate = self.request.localizer.translate
        if changes:
            message = translate(_("Request successful, {count} new notifications "
                                  "have been sent.")).format(count=changes)
        else:
            message = translate(_("Request successful, no new notification have been sent."))
        comments = IReviewComments(self.context)
        return {
            'status': 'reload',
            'smallbox': {
                'status': 'success',
                'message': message
            },
            'events': [{
                'event': 'update-comments.ams.content',
                'options': {
                    'count': len(comments)
                }
            }]
        }


#
# Share contents comments
#

@viewlet_config(name='review-comments.menu',
                context=IReviewTarget, layer=IAdminLayer,
                manager=IContentManagementMenu, weight=30,
                permission=COMMENT_CONTENT_PERMISSION)
class ContentReviewCommentsMenu(NavigationMenuItem):
    """Content review comments menu"""

    label = _("Comments")
    icon_class = 'far fa-comments'

    href = '#review-comments.html'

    badge_status = 'info'

    @property
    def object_data(self):
        """Object data getter"""
        return {
            'ams-modules': {
                'content': {
                    'src': get_resource_path(content_js)
                }
            },
            'ams-callback': 'MyAMS.content.review.init'
        }

    def update(self):
        super().update()
        nb_comments = len(IReviewComments(self.context))
        self.badge = str(nb_comments)


@pagelet_config(name='review-comments.html',
                context=IReviewTarget, layer=IPyAMSLayer,
                permission=COMMENT_CONTENT_PERMISSION)
@template_config(template='templates/comments.pt', layer=IAdminLayer)
class ContentReviewCommentsView(InnerAdminView):
    """Content review comments view"""

    title = _("Version comments")

    comments = None
    security = None

    def update(self):
        super().update()
        self.comments = IReviewComments(self.context).values()
        self.security = get_utility(ISecurityManager)

    def get_principal(self, principal_id):
        """Principal getter"""
        return self.security.get_principal(principal_id)

    @staticmethod
    def get_avatar(principal):
        """Principal avatar getter"""
        return IUserProfile(principal).get_avatar()

    @staticmethod
    def get_date(comment):
        """Comment date getter"""
        return format_datetime(comment.creation_date)

    @staticmethod
    def get_age(comment):
        """Comment age getter"""
        return get_age(comment.creation_date)


@view_config(name='add-review-comment.json',
             context=IReviewTarget, request_type=IPyAMSLayer,
             renderer='json', xhr=True,
             permission=COMMENT_CONTENT_PERMISSION)
def add_review_comment(request):
    """Review comment add view"""
    translate = request.localizer.translate
    comment_body = request.params.get('comment')
    if not comment_body:
        return {
            'status': 'error',
            'message': translate(_("Message is mandatory!"))
        }
    # add new comment
    comments = IReviewComments(request.context)
    factory = get_object_factory(IReviewComment)
    comment = factory(owner=request.principal.id,
                      comment=request.params.get('comment'))
    roles = IWfSharedContentRoles(request.context, None)
    if roles is not None:
        comment.is_reviewer_comment = comment.owner in (roles.readers or ())
    comments.add_comment(comment)
    # return comment infos
    profile = IUserProfile(request.principal)
    comment_body = render('templates/comment.pt', request=request, value={
        'comment': comment,
        'comment_date': format_datetime(comment.creation_date),
        'comment_age': get_age(comment.creation_date, request=request),
        'profile': profile
    })
    return {
        'status': 'success',
        'callbacks': [{
            'callback': 'MyAMS.helpers.addElementToParent',
            'options': {
                'element': comment_body,
                'parent': '#review-messages',
                'scrollTo': True,
                'scrollParent': '#review-messages-view'
            },
        }],
        'events': [{
            'event': 'update-comments.ams.content',
            'options': {
                'count': len(comments)
            }
        }]
    }


@view_config(name='get-comments.json',
             context=IReviewTarget, request_type=IPyAMSLayer,
             renderer='json',
             permission=COMMENT_CONTENT_PERMISSION)
def get_review_comments(request):
    """Review comments getter"""
    count = int(request.params.get('count', 0))
    comments = IReviewComments(request.context)
    if len(comments) <= count:
        return {
            'status': 'success',
            'count': count
        }
    result = []
    for index, comment in enumerate(comments.values()):
        if index < count:
            continue
        principal = get_principal(request, principal_id=comment.owner)
        result.append(render('templates/comment.pt', request=request, value={
            'comment': comment,
            'comment_date': format_datetime(comment.creation_date),
            'comment_age': get_age(comment.creation_date, request=request),
            'profile': IUserProfile(principal)
        }))
    return {
        'status': 'success',
        'comments': result,
        'count': len(comments)
    }



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.extfile.zmi.manager module

This module provides management components to external files manager.
"""

from pyams_content.component.extfile import IExtFileManagerInfo
from pyams_content.component.extfile.interfaces import IExtFileManagerTarget
from pyams_content.interfaces import MANAGE_SITE_ROOT_PERMISSION
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IFormContent
from pyams_layer.interfaces import IPyAMSLayer
from pyams_utils.adapter import adapter_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IPropertiesMenu
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem


__docformat__ = 'restructuredtext'

from pyams_content import _


@viewlet_config(name='extfile-manager.menu',
                context=IExtFileManagerTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=770,
                permission=MANAGE_SITE_ROOT_PERMISSION)
class ExtFileManagerMenu(NavigationMenuItem):
    """External files manager menu"""

    label = _("External files settings")
    href = '#extfile-manager.html'


@ajax_form_config(name='extfile-manager.html',
                  context=IExtFileManagerTarget, layer=IPyAMSLayer,
                  permission=MANAGE_SITE_ROOT_PERMISSION)
class ExtFileManagerPropertiesEditForm(AdminEditForm):
    """External files manager properties edit form"""

    title = _("External files settings")
    legend = _("Default prefix")

    fields = Fields(IExtFileManagerInfo)


@adapter_config(required=(IExtFileManagerTarget, IPyAMSLayer, ExtFileManagerPropertiesEditForm),
                provides=IFormContent)
def extfile_manager_properties_form_content(context, request, form):
    """External file manager properties edit form content getter"""
    return IExtFileManagerInfo(context)



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.thesaurus module

This module handles persistent classes using thesaurus based properties.
"""

from persistent import Persistent
from pyramid.events import subscriber
from pyramid.location import lineage
from zope.container.contained import Contained
from zope.interface import alsoProvides
from zope.lifecycleevent import IObjectAddedEvent, IObjectModifiedEvent, ObjectModifiedEvent
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.thesaurus.interfaces import COLLECTIONS_INFO_KEY, \
    COLLECTIONS_MANAGER_KEY, ICollectionsInfo, ICollectionsManager, ICollectionsManagerTarget, \
    ICollectionsTarget, ITagsInfo, ITagsManager, ITagsManagerTarget, ITagsTarget, IThemesInfo, \
    IThemesManager, IThemesManagerTarget, IThemesTarget, TAGS_INFO_KEY, TAGS_MANAGER_KEY, \
    THEMES_INFO_KEY, THEMES_MANAGER_KEY
from pyams_content.shared.site.interfaces import ISiteFolder
from pyams_security.interfaces import IViewContextPermissionChecker
from pyams_security.permission import get_edit_permission
from pyams_thesaurus.interfaces.thesaurus import IThesaurus
from pyams_utils.adapter import ContextAdapter, adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.inherit import BaseInheritInfo, InheritedFieldProperty
from pyams_utils.registry import get_pyramid_registry, query_utility
from pyams_utils.request import query_request
from pyams_utils.traversing import get_parent


__docformat__ = 'restructuredtext'


@adapter_config(required=ITagsInfo,
                provides=IViewContextPermissionChecker)
@adapter_config(required=IThemesInfo,
                provides=IViewContextPermissionChecker)
@adapter_config(required=ICollectionsInfo,
                provides=IViewContextPermissionChecker)
class TagsInfoPermissionChecker(ContextAdapter):
    """Tags info permission checker"""

    @property
    def edit_permission(self):
        """Tags info edit permission getter"""
        request = query_request()
        for index, parent in enumerate(lineage(self.context)):
            if index == 0:
                continue
            permission = get_edit_permission(request, parent)
            if permission is not None:
                return permission
        return None


#
# Tags management
#

@factory_config(ITagsManager)
class TagsManager(Persistent, Contained):
    """Tags manager persistent class"""

    thesaurus_name = FieldProperty(ITagsManager['thesaurus_name'])
    extract_name = FieldProperty(ITagsManager['extract_name'])

    enable_glossary = FieldProperty(ITagsManager['enable_glossary'])
    glossary_thesaurus_name = FieldProperty(ITagsManager['glossary_thesaurus_name'])

    @property
    def glossary(self):
        """Glossary getter"""
        if not self.enable_glossary:
            return None
        return query_utility(IThesaurus, name=self.glossary_thesaurus_name)


@adapter_config(required=ITagsManagerTarget,
                provides=ITagsManager)
def tags_manager_factory(target):
    """Tags manager factory"""
    return get_annotation_adapter(target, TAGS_MANAGER_KEY, ITagsManager,
                                  name='++tags-manager++')


@factory_config(ITagsInfo)
class TagsInfo(Persistent, Contained):
    """Tags info persistent class"""

    tags = FieldProperty(ITagsInfo['tags'])


@adapter_config(required=ITagsTarget,
                provides=ITagsInfo)
def tags_info_factory(target):
    """Tags info factory"""
    return get_annotation_adapter(target, TAGS_INFO_KEY, ITagsInfo,
                                  name='++tags++')


@subscriber(IObjectModifiedEvent, context_selector=ITagsInfo)
def handle_modified_tags_info(event):
    """Handle modified tags info"""
    target = get_parent(event.object, ITagsTarget)
    if target is not None:
        registry = get_pyramid_registry()
        registry.notify(ObjectModifiedEvent(target))


#
# Themes management
#

@factory_config(IThemesManager)
class ThemesManager(Persistent, Contained):
    """Themes manager persistent class"""

    thesaurus_name = FieldProperty(IThemesManager['thesaurus_name'])
    extract_name = FieldProperty(IThemesManager['extract_name'])


@adapter_config(required=IThemesManagerTarget,
                provides=IThemesManager)
def themes_manager_factory(target):
    """Themes manager factory"""
    return get_annotation_adapter(target, THEMES_MANAGER_KEY, IThemesManager,
                                  name='++themes-manager++')


@factory_config(IThemesInfo)
class ThemesInfo(BaseInheritInfo, Persistent, Contained):
    """Themes info persistent class"""

    adapted_interface = IThemesInfo
    target_interface = IThemesTarget

    _themes = FieldProperty(IThemesInfo['themes'])
    themes = InheritedFieldProperty(IThemesInfo['themes'])


@adapter_config(required=IThemesTarget,
                provides=IThemesInfo)
def themes_info_factory(target):
    """Themes info factory"""
    return get_annotation_adapter(target, THEMES_INFO_KEY, IThemesInfo,
                                  name='++themes++')


@subscriber(IObjectAddedEvent, context_selector=ISiteFolder, parent_selector=IThemesTarget)
def handle_added_site_folder(event):
    """Handle site folder when added to a themes target"""
    alsoProvides(event.object, IThemesTarget)


@subscriber(IObjectModifiedEvent, context_selector=IThemesInfo)
def handle_modified_themes_info(event):
    """Handle modified themes info"""
    target = get_parent(event.object, IThemesTarget)
    if target is not None:
        registry = get_pyramid_registry()
        registry.notify(ObjectModifiedEvent(target))


#
# Collections management
#

@factory_config(ICollectionsManager)
class CollectionsManager(Persistent, Contained):
    """Collections manager persistent class"""

    thesaurus_name = FieldProperty(ICollectionsManager['thesaurus_name'])
    extract_name = FieldProperty(ICollectionsManager['extract_name'])


@adapter_config(required=ICollectionsManagerTarget,
                provides=ICollectionsManager)
def collections_manager_factory(target):
    """Collections manager factory"""
    return get_annotation_adapter(target, COLLECTIONS_MANAGER_KEY, ICollectionsManager,
                                  name='++collections-manager++')


@factory_config(ICollectionsInfo)
class CollectionsInfo(Persistent, Contained):
    """Collections info persistent class"""

    collections = FieldProperty(ICollectionsInfo['collections'])


@adapter_config(required=ICollectionsTarget,
                provides=ICollectionsInfo)
def collections_info_factory(target):
    """Collections info factory"""
    return get_annotation_adapter(target, COLLECTIONS_INFO_KEY, ICollectionsInfo,
                                  name='++collections++')


@subscriber(IObjectModifiedEvent, context_selector=ICollectionsInfo)
def handle_modified_collections_info(event):
    """Handle modified collections info"""
    target = get_parent(event.object, ICollectionsTarget)
    if target is not None:
        registry = get_pyramid_registry()
        registry.notify(ObjectModifiedEvent(target))



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.site.topic module

"""

from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.illustration import IIllustrationTarget, ILinkIllustrationTarget
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget
from pyams_content.component.thesaurus import ITagsTarget, IThemesTarget
from pyams_content.feature.preview.interfaces import IPreviewTarget
from pyams_content.feature.review import IReviewTarget
from pyams_content.shared.common import ISharedContent, SharedContent, WfSharedContent
from pyams_content.shared.common.interfaces import IWfSharedContent
from pyams_content.shared.common.types import WfTypedSharedContentMixin
from pyams_content.shared.site.interfaces import ISiteTopic, IWfSiteTopic, \
    SITE_TOPIC_CONTENT_NAME, SITE_TOPIC_CONTENT_TYPE
from pyams_portal.interfaces import IPortalContext, IPortalFooterContext, IPortalHeaderContext
from pyams_utils.factory import factory_config
from pyams_workflow.interfaces import IWorkflow, IWorkflowState, IWorkflowVersions


__docformat__ = 'restructuredtext'


@factory_config(IWfSiteTopic)
@factory_config(IWfSharedContent, name=SITE_TOPIC_CONTENT_TYPE)
@implementer(IIllustrationTarget, ILinkIllustrationTarget,
             IParagraphContainerTarget, ITagsTarget, IThemesTarget,
             IPortalContext, IPortalHeaderContext, IPortalFooterContext,
             IReviewTarget, IPreviewTarget)
class WfSiteTopic(WfSharedContent, WfTypedSharedContentMixin):
    """Base site topic"""

    content_type = SITE_TOPIC_CONTENT_TYPE
    content_name = SITE_TOPIC_CONTENT_NAME
    content_intf = IWfSiteTopic
    content_view = True

    references = FieldProperty(IWfSiteTopic['references'])
    data_type = FieldProperty(IWfSiteTopic['data_type'])


@factory_config(ISiteTopic)
@factory_config(ISharedContent, name=SITE_TOPIC_CONTENT_TYPE)
class SiteTopic(SharedContent):
    """Workflow managed topic class"""

    content_type = SITE_TOPIC_CONTENT_TYPE
    content_name = SITE_TOPIC_CONTENT_NAME

    def is_deletable(self):
        """Topic deletion checker"""
        workflow = IWorkflow(self)
        for version in IWorkflowVersions(self).get_versions():
            if IWorkflowState(version).state != workflow.initial_state:
                return False
        return True



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.association.paragraph module

This module defines a paragraph factory which is only used to store associations.
"""

from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.association.interfaces import ASSOCIATION_PARAGRAPH_ICON_CLASS, \
    ASSOCIATION_PARAGRAPH_NAME, ASSOCIATION_PARAGRAPH_RENDERERS, ASSOCIATION_PARAGRAPH_TYPE, \
    IAssociationParagraph
from pyams_content.component.extfile.interfaces import IExtFileContainerTarget
from pyams_content.component.links.interfaces import ILinkContainerTarget
from pyams_content.component.paragraph import BaseParagraph, IBaseParagraph
from pyams_content.feature.renderer import RenderersVocabulary
from pyams_utils.factory import factory_config
from pyams_utils.vocabulary import vocabulary_config


__docformat__ = 'restructuredtext'


@factory_config(IAssociationParagraph)
@factory_config(IBaseParagraph, name=ASSOCIATION_PARAGRAPH_TYPE)
@implementer(IExtFileContainerTarget, ILinkContainerTarget)
class AssociationParagraph(BaseParagraph):
    """Associations paragraph"""

    factory_name = ASSOCIATION_PARAGRAPH_TYPE
    factory_label = ASSOCIATION_PARAGRAPH_NAME
    factory_intf = IAssociationParagraph

    icon_class = ASSOCIATION_PARAGRAPH_ICON_CLASS

    renderer = FieldProperty(IAssociationParagraph['renderer'])


@vocabulary_config(name=ASSOCIATION_PARAGRAPH_RENDERERS)
class AssociationParagraphRenderersVocabulary(RenderersVocabulary):
    """Associations paragraph renderers vocabulary"""

    content_interface = IAssociationParagraph



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.review module

"""

from datetime import datetime
from uuid import uuid4

from persistent import Persistent
from pyramid.events import subscriber
from zope.container.contained import Contained
from zope.lifecycleevent import IObjectCreatedEvent
from zope.location.interfaces import ISublocations
from zope.schema.fieldproperty import FieldProperty
from zope.traversing.interfaces import ITraversable

from pyams_content.feature.review.interfaces import CommentAddedEvent, ICommentAddedEvent, \
    IReviewComment, IReviewComments, IReviewManager, IReviewTarget, REVIEW_COMMENTS_ANNOTATION_KEY
from pyams_security.interfaces import ISecurityManager
from pyams_utils.adapter import ContextAdapter, adapter_config, get_annotation_adapter
from pyams_utils.container import BTreeOrderedContainer
from pyams_utils.factory import factory_config
from pyams_utils.registry import get_pyramid_registry, get_utility


__docformat__ = 'restructuredtext'

from pyams_workflow.interfaces import IObjectClonedEvent


@factory_config(IReviewComment)
class ReviewComment(Persistent, Contained):
    """Review comment persistent class"""

    owner = FieldProperty(IReviewComment['owner'])
    reviewers = FieldProperty(IReviewComment['reviewers'])
    comment_type = FieldProperty(IReviewComment['comment_type'])
    comment = FieldProperty(IReviewComment['comment'])
    is_reviewer_comment = FieldProperty(IReviewComment['is_reviewer_comment'])
    creation_date = FieldProperty(IReviewComment['creation_date'])

    def __init__(self, owner, comment, comment_type='comment', reviewers=None):
        self.owner = owner
        self.comment = comment
        self.comment_type = comment_type
        sm = get_utility(ISecurityManager)
        if reviewers:
            self.reviewers = ', '.join((
                principal.title
                for principal in (sm.get_principal(reviewer) for reviewer in reviewers)
            ))
        self.creation_date = datetime.utcnow()


@factory_config(IReviewComments)
class ReviewCommentsContainer(BTreeOrderedContainer):
    """Review comments container"""

    reviewers = FieldProperty(IReviewComments['reviewers'])

    def clear(self):
        for k in self.keys()[:]:
            del self[k]

    def add_comment(self, comment):
        uuid = str(uuid4())
        self[uuid] = comment
        reviewers = self.reviewers or set()
        reviewers.add(comment.owner)
        self.reviewers = reviewers
        get_pyramid_registry().notify(CommentAddedEvent(self.__parent__, comment))


@adapter_config(required=IReviewTarget,
                provides=IReviewComments)
def review_comments_factory(context):
    """Review comments factory"""
    return get_annotation_adapter(context, REVIEW_COMMENTS_ANNOTATION_KEY, IReviewComments,
                                  name='++review-comments++')


@adapter_config(name='review-comments',
                required=IReviewTarget,
                provides=ITraversable)
class ReviewCommentsNamespace(ContextAdapter):
    """++review-comments++ namespace traverser"""

    def traverse(self, name, furtherpath=None):
        return IReviewComments(self.context)


@adapter_config(name='review-comments',
                required=IReviewTarget,
                provides=ISublocations)
class ReviewCommentsSublocations(ContextAdapter):
    """Review comments sub-location adapter"""

    def sublocations(self):
        """Sub-locations iterator"""
        yield from IReviewComments(self.context).values()


@subscriber(IObjectCreatedEvent, context_selector=IReviewTarget)
@subscriber(IObjectClonedEvent, context_selector=IReviewTarget)
def clone_review_target(event):
    """Clear comments when a review target is cloned"""
    comments = IReviewComments(event.object, None)
    if comments is not None:
        comments.clear()



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.video.skin.provider module

This module defines base external videos renderers class.
"""

__docformat__ = 'restructuredtext'

from urllib.parse import urlencode

from pyams_content.component.video.provider import ICustomVideoSettings
from pyams_content.component.video.skin import IExternalVideoRenderer
from pyams_layer.interfaces import IPyAMSLayer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_viewlet.viewlet import BaseContentProvider


def time_to_seconds(value):
    """Convert min:sec value to seconds"""
    if value and (':' in value):
        min, sec = value.split(':', 1)
        return str(int(min)*60 + int(sec))
    return value or ''


def get_playlist_id(settings):
    """Include playlist ID if loop is required"""
    if settings.loop:
        return settings.video_id
    return None


class BaseExternalVideoRenderer(BaseContentProvider):
    """Base external video renderer"""

    params = ()

    def get_url_params(self):
        settings = self.context
        params = {}
        for attr, param, handler in self.params:
            if attr is None:
                result = handler(settings)
            else:
                result = handler(getattr(settings, attr))
            if result is not None:
                params[param] = result
        return urlencode(params)


@adapter_config(required=(ICustomVideoSettings, IPyAMSLayer),
                provides=IExternalVideoRenderer)
@template_config(template='templates/custom-render.pt', layer=IPyAMSLayer)
class CustomVideoRenderer(BaseExternalVideoRenderer):
    """Custom video renderer"""



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.interfaces module

This module defines interfaces which are common to all shared contents.
"""

from zope.annotation import IAttributeAnnotatable
from zope.container.constraints import containers, contains
from zope.container.interfaces import IContainer
from zope.interface import Attribute, Interface
from zope.schema import Bool, Choice, Text, TextLine

from pyams_content.interfaces import CONTRIBUTOR_ROLE, GUEST_ROLE, IBaseContent, MANAGER_ROLE, \
    OWNER_ROLE, PILOT_ROLE, READER_ROLE, WEBMASTER_ROLE
from pyams_i18n.schema import I18nTextField
from pyams_portal.interfaces import DESIGNER_ROLE, IPortalContext, IPortalPage
from pyams_security.schema import PrincipalField, PrincipalsSetField
from pyams_site.interfaces import ISiteRoot
from pyams_utils.schema import TextLineListField
from pyams_workflow.interfaces import IWorkflowManagedContent, WORKFLOWS_VOCABULARY


__docformat__ = 'restructuredtext'

from pyams_content import _


class IDeletableElement(Interface):
    """Deletable element interface"""

    def is_deletable(self):
        """Check to know if a site element can be deleted"""


CONTENT_MANAGER_ROLES = 'pyams_content.manager.roles'
CONTENT_MANAGER_CONTRIBUTORS = 'pyams_content.manager.roles::contributor'


class IBaseContentManagerRoles(Interface):
    """Shared tool roles interface"""

    webmasters = PrincipalsSetField(title=_("Webmasters"),
                                    description=_("Webmasters can handle all contents, including "
                                                  "published ones"),
                                    role_id=WEBMASTER_ROLE,
                                    required=False)

    pilots = PrincipalsSetField(title=_("Pilots"),
                                description=_("Pilots can handle tool configuration, manage "
                                              "access rules, grant users roles and manage "
                                              "managers restrictions"),
                                role_id=PILOT_ROLE,
                                required=False)

    managers = PrincipalsSetField(title=_("Managers"),
                                  description=_("Managers can handle main operations in tool's "
                                                "workflow, like publish or retire contents"),
                                  role_id=MANAGER_ROLE,
                                  required=False)

    contributors = PrincipalsSetField(title=_("Contributors"),
                                      description=_("Contributors are users which are allowed to "
                                                    "create new contents"),
                                      role_id=CONTRIBUTOR_ROLE,
                                      required=False)

    designers = PrincipalsSetField(title=_("Designers"),
                                   description=_("Designers are users which are allowed to "
                                                 "manage presentation templates"),
                                   role_id=DESIGNER_ROLE,
                                   required=False)


class ISharedSite(IBaseContent, IDeletableElement):
    """Shared site interface"""

    containers(ISiteRoot)

    content_name = Attribute("Site content name")


class ISharedToolContainer(IBaseContent, IContainer):
    """Shared tools container"""

    containers(ISiteRoot)
    contains('.ISharedTool')


DEFAULT_CONTENT_WORKFLOW = 'pyams_content.workflow.default'
BASIC_CONTENT_WORKFLOW = 'pyams_content.workflow.basic'


class IBaseSharedTool(IBaseContent, IContainer):
    """Base shared tool interface"""

    containers(ISharedToolContainer)

    shared_content_menu = Attribute("Boolean flag indicating if tool is displayed into 'Shared "
                                    "contents' or Shared tools' menu")

    shared_content_workflow = Choice(title=_("Workflow name"),
                                     description=_("Name of workflow utility used to manage tool "
                                                   "contents"),
                                     vocabulary=WORKFLOWS_VOCABULARY,
                                     default=DEFAULT_CONTENT_WORKFLOW)


SHARED_TOOL_WORKFLOW_STATES_VOCABULARY = 'PyAMS workflow states'


class ISharedTool(IBaseSharedTool):
    """Shared tool interface"""

    contains('.ISharedContent')

    shared_content_type = Attribute("Shared data content type name")


class ISharedToolPortalContext(ISharedTool, IPortalContext):
    """Shared tool with portal context"""


class ISharedToolRoles(IBaseContentManagerRoles):
    """Shared tool roles"""


class IWfSharedContent(IBaseContent):
    """Shared content interface"""

    content_type = Attribute("Content data type")
    content_name = Attribute("Content name")
    content_intf = Attribute("Content interface")
    content_view = Attribute("Available for views searching")

    content_url = TextLine(title=_("Content URL"),
                           description=_("URL used to access this content; this is important for "
                                         "SEO and should include most important words describing "
                                         "content; spaces and underscores will be automatically "
                                         "replaced by hyphens"),
                           required=True)

    creator = PrincipalField(title=_("Version creator"),
                             description=_("Name of content's version creator. "
                                           "The creator of the first version is also it's "
                                           "owner."),
                             required=True)

    first_owner = PrincipalField(title=_("First owner"),
                                 description=_("Name of content's first version owner"),
                                 required=True,
                                 readonly=True)

    creation_label = TextLine(title=_("Version creation"),
                              readonly=True)

    modifiers = PrincipalsSetField(title=_("Version modifiers"),
                                   description=_("List of principals who modified this content"),
                                   required=False)

    last_modifier = PrincipalField(title=_("Last modifier"),
                                   description=_("Last principal who modified this content"),
                                   required=False)

    last_update_label = TextLine(title=_("Last update"),
                                 readonly=True)

    header = I18nTextField(title=_("Header"),
                           description=_("Content's header is generally displayed in page "
                                         "header"),
                           required=False)

    description = I18nTextField(title=_("Meta-description"),
                                description=_("The content's description is 'hidden' into HTML's "
                                              "page headers; but it can be seen, for example, in "
                                              "some search engines results as content's "
                                              "description; if description is empty, content's "
                                              "header will be used."),
                                required=False)

    keywords = TextLineListField(title=_("Keywords"),
                                 description=_("They will be included into HTML pages metadata"),
                                 required=False)

    notepad = Text(title=_("Notepad"),
                   description=_("Internal information to be known about this content"),
                   required=False)


class IPreventSharedContentUpdateSubscribers(Interface):
    """Shared content update subscribers handler"""


class IBaseContentPortalContext(IPortalContext):
    """Content portal context interface"""


class IWfSharedContentPortalContext(IWfSharedContent, IBaseContentPortalContext):
    """Shared content with portal support"""


class ISharedContentPortalPage(IPortalPage):
    """Shared content portal page interface"""


SHARED_CONTENT_ROLES = 'pyams_content.content.roles'


class IWfSharedContentRoles(Interface):
    """Shared content roles"""

    owner = PrincipalsSetField(title=_("Content owner"),
                               description=_("The owner is the creator of content's first "
                                             "version, except if it was transferred afterwards "
                                             "to another owner"),
                               role_id=OWNER_ROLE,
                               required=True,
                               max_length=1)

    managers = PrincipalsSetField(title=_("Managers"),
                                  description=_("Managers can handle main operations in tool's "
                                                "workflow, like publish or retire contents"),
                                  role_id=MANAGER_ROLE,
                                  required=False)

    contributors = PrincipalsSetField(title=_("Contributors"),
                                      description=_("Contributors are users which are allowed "
                                                    "to update this content in addition to "
                                                    "it's owner"),
                                      role_id=CONTRIBUTOR_ROLE,
                                      required=False)

    designers = PrincipalsSetField(title=_("Designers"),
                                   description=_("Designers are users which are allowed to "
                                                 "manage presentation templates"),
                                   role_id=DESIGNER_ROLE,
                                   required=False)

    readers = PrincipalsSetField(title=_("Readers"),
                                 description=_("Readers are users which are asked to verify and "
                                               "comment contents before they are published"),
                                 role_id=READER_ROLE,
                                 required=False)

    guests = PrincipalsSetField(title=_("Guests"),
                                description=_("Guests are users which are allowed to view "
                                              "contents with restricted access"),
                                role_id=GUEST_ROLE,
                                required=False)


class ISharedContent(IWorkflowManagedContent):
    """Workflow managed shared content interface"""

    content_type = Attribute("Content type interface")
    content_name = Attribute("Content name")
    content_factory = Attribute("Content factory attribute")
    content_view = Attribute("Available for views searching")

    visible_version = Attribute("Link to actually visible version")


CONTENT_TYPES_VOCABULARY = 'pyams_content.content.types'

SHARED_CONTENT_TYPES_VOCABULARY = 'pyams_content.shared_content.types'

VIEWS_SHARED_CONTENT_TYPES_VOCABULARY = 'pyams_content.shared_content.types.views'


#
# Generic restrictions interfaces
#

class ISharedToolRestrictions(Interface):
    """Base shared tool restrictions interface"""

    def new_restrictions(self, principal):
        """Create new restrictions for given principal"""

    def set_restrictions(self, principal, restrictions=None, interface=None):
        """Set restrictions for given principal"""

    def get_restrictions(self, principal, create_if_empty=False):
        """Get restrictions for given principal"""

    def drop_restrictions(self, principal):
        """Drop restrictions for given principal"""

    def can_access(self, context, permission, request=None):
        """Check if given request can get access to context with given permission"""


class IPrincipalRestrictions(IAttributeAnnotatable):
    """Principal restrictions base class"""

    principal_id = PrincipalField(title=_("Principal ID"),
                                  required=True)


class IRestrictionInfo(Interface):
    """Base restriction interface"""

    weight = Attribute("Adapter ordering weight")

    def can_access(self, context, permission, request):
        """Check if given context can be accessed with given permission"""


#
# Contributors restrictions
#

CONTRIBUTORS_RESTRICTIONS_KEY = 'pyams_content.restrictions.contributors'


class IContributorRestrictions(ISharedToolRestrictions):
    """Contributor restrictions marker interface"""


CONTRIBUTOR_WORKFLOW_RESTRICTIONS_KEY = f'{CONTRIBUTORS_RESTRICTIONS_KEY}::workflow'


class IContributorWorkflowRestrictions(IRestrictionInfo):
    """Base contributor workflow restrictions interface"""

    show_workflow_warning = Bool(title=_("Show workflow checks warning"),
                                 description=_("If 'yes', this contributor will have to confirm "
                                               "that contents have been checked before asking "
                                               "for publication"),
                                 required=True,
                                 default=True)

    owners = PrincipalsSetField(title=_("Substitute for"),
                                description=_("Contributor will have access to contents owned "
                                              "by these principals"),
                                required=False)


#
# Managers restrictions
#

MANAGERS_RESTRICTIONS_KEY = 'pyams_content.restrictions.managers'


class IManagerRestrictions(ISharedToolRestrictions):
    """Manager restrictions marker interface"""


MANAGER_WORKFLOW_RESTRICTIONS_KEY = f'{MANAGERS_RESTRICTIONS_KEY}::workflow'


class IManagerWorkflowRestrictions(IRestrictionInfo):
    """Base manager workflow restrictions interface"""

    show_workflow_warning = Bool(title=_("Publication checks"),
                                 description=_("If 'yes', this manager will have to confirm that "
                                               "contents have been previewed and checked before "
                                               "publishing a content"),
                                 required=True,
                                 default=True)

    restricted_contents = Bool(title=_("Restricted contents"),
                               description=_("If 'yes', this manager will get restricted access "
                                             "to manage contents based on selected settings"),
                               required=True,
                               default=True)

    owners = PrincipalsSetField(title=_("Selected owners"),
                                description=_("Manager will have access to contents owned "
                                              "by these principals"),
                                required=False)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from datetime import datetime

from hypatia.interfaces import ICatalog
from hypatia.catalog import CatalogQuery
from hypatia.query import And, Eq, Any, Lt, Or, Gt
from pyramid.events import subscriber
from zope.component.interfaces import ISite
from zope.lifecycleevent.interfaces import IObjectAddedEvent

from pyams_catalog.query import CatalogResultSet
from pyams_content.shared.alert.interfaces import ALERT_CONTENT_TYPE, IAlertManager
from pyams_content.shared.common.manager import SharedTool
from pyams_sequence.interfaces import ISequentialIdInfo
from pyams_utils.factory import factory_config
from pyams_utils.registry import get_utility
from pyams_utils.request import check_request
from pyams_utils.timezone import tztime
from pyams_utils.traversing import get_parent
from pyams_workflow.interfaces import IWorkflow


@factory_config(IAlertManager)
class AlertManager(SharedTool):
    """Alert manager class"""

    shared_content_type = ALERT_CONTENT_TYPE
    shared_content_menu = False

    def find_context_alerts(self, context=None, request=None):
        """Find alerts matching given params"""
        if request is None:
            request = check_request()
        if context is None:
            context = request.context
        sequence_info = ISequentialIdInfo(context, None)
        if sequence_info is None:
            return
        now = tztime(datetime.utcnow())
        catalog = get_utility(ICatalog)
        workflow = IWorkflow(self)
        params = And(Eq(catalog['content_type'], self.shared_content_type),
                     Eq(catalog['link_references'], sequence_info.hex_oid),
                     Any(catalog['workflow_state'], workflow.published_states),
                     Lt(catalog['effective_date'], now),
                     Or(Gt(catalog['push_end_date'], now),
                        Eq(catalog['push_end_date'], None)))
        yield from CatalogResultSet(CatalogQuery(catalog).query(params))


@subscriber(IObjectAddedEvent, context_selector=IAlertManager)
def handle_added_alert_manager(event):
    """Register alert manager when added"""
    site = get_parent(event.newParent, ISite)
    registry = site.getSiteManager()
    if registry is not None:
        registry.registerUtility(event.object, IAlertManager)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.search.portlet module

"""

import math
from itertools import tee

from zope.schema.fieldproperty import FieldProperty

from pyams_content.feature.search import ISearchFolder
from pyams_content.feature.search.portlet.interfaces import IAggregatedPortletRenderer, \
    ISearchResultsPortletSettings, SEARCH_RESULTS_ICON_CLASS, SEARCH_RESULTS_PORTLET_FLAG, SEARCH_RESULTS_PORTLET_NAME
from pyams_content.shared.view.interfaces import RELEVANCE_ORDER, VISIBLE_PUBLICATION_DATE_ORDER
from pyams_portal.interfaces import IPortletRendererSettings
from pyams_portal.portlet import Portlet, PortletSettings, portlet_config
from pyams_utils.factory import factory_config
from pyams_utils.list import boolean_iter
from pyams_utils.request import check_request
from pyams_utils.traversing import get_parent

__docformat__ = 'restructuredtext'

from pyams_content import _


@factory_config(ISearchResultsPortletSettings)
class SearchResultsPortletSettings(PortletSettings):
    """Search results portlet settings"""

    title = FieldProperty(ISearchResultsPortletSettings['title'])
    allow_empty_query = FieldProperty(ISearchResultsPortletSettings['allow_empty_query'])
    force_canonical_url = FieldProperty(ISearchResultsPortletSettings['force_canonical_url'])

    @staticmethod
    def has_user_query(request):
        """User query checker"""
        return bool(request.params.get('user_search', '').strip())

    def _get_items(self, request=None, start=0, length=10, limit=None, ignore_cache=False):
        context = get_parent(request.context, ISearchFolder)
        if context is None:
            raise StopIteration
        else:
            if request is None:
                request = check_request()
            params = request.params
            order_by = params.get('order_by', context.order_by)
            if (order_by == RELEVANCE_ORDER) and \
                    not self.has_user_query(request):
                request.GET['order_by'] = order_by = VISIBLE_PUBLICATION_DATE_ORDER
            renderer_settings = IPortletRendererSettings(self)
            if IAggregatedPortletRenderer.providedBy(renderer_settings):
                aggregates = renderer_settings.aggregates
                ignore_cache = True
            else:
                aggregates = {}
            yield from context.get_results(context, order_by,
                                           reverse=order_by != RELEVANCE_ORDER,
                                           limit=limit,
                                           start=int(start),
                                           length=int(length),
                                           ignore_cache=ignore_cache,
                                           get_count=True,
                                           request=request,
                                           aggregates=aggregates,
                                           settings=self)

    def get_items(self, request=None, start=0, length=10, limit=None, ignore_cache=False):
        """Search results getter"""
        if not (self.allow_empty_query or self.has_user_query(request)):
            yield from iter(((), 0, {}), )
        else:
            has_items, items = boolean_iter(self._get_items(request, start, length, limit,
                                                            ignore_cache))
            if not has_items:
                yield from iter(((), 0, {}), )
            else:
                # verify real items count
                check, items = tee(items)
                _values = next(check)
                count = next(check)
                if count:
                    if request is None:
                        request = check_request()
                    request.annotations[SEARCH_RESULTS_PORTLET_FLAG] = True
                yield from items

    @staticmethod
    def get_pages(start, length, count):
        start = int(start) + 1
        length = int(length)
        current = math.ceil(start / length)
        nb_pages = math.ceil(count / length)
        return current, nb_pages


@portlet_config(permission=None)
class SearchResultsPortlet(Portlet):
    """Search folder results portlet"""

    name = SEARCH_RESULTS_PORTLET_NAME
    label = _("Search results")

    settings_factory = ISearchResultsPortletSettings
    toolbar_css_class = SEARCH_RESULTS_ICON_CLASS



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'


from zope.contentprovider.interfaces import IContentProvider

from pyams_content.component.paragraph.interfaces import IBaseParagraph, IParagraphContainer
from pyams_content.component.paragraph.zmi.interfaces import IParagraphContainerFullTable
from pyams_utils.factory import get_object_factory
from pyams_utils.traversing import get_parent


def get_json_paragraph_toolbar_refresh_event(context, request):
    """Get paragraph toolbar refresh event"""
    paragraph = get_parent(context, IBaseParagraph)
    if paragraph is None:
        return None
    container = get_parent(paragraph, IParagraphContainer)
    if container is None:
        return None
    factory = get_object_factory(IParagraphContainerFullTable)
    if factory is None:
        return None
    table = factory(container, request)
    provider = request.registry.queryMultiAdapter((paragraph, request, table), IContentProvider,
                                                  name='pyams_content.paragraph.title-toolbar')
    if provider is None:
        return None
    provider.update()
    return {
        'callback': 'MyAMS.helpers.refreshElement',
        'options': {
            'object_id': provider.id,
            'content': provider.render()
        }
    }
